////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv;

import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import edu.uiuc.jdbv.util.SuffixFileFilter;

import java.beans.XMLEncoder;
import edu.uiuc.jdbv.diagram.xmlPersist.JDBVXMLEncoder;

import edu.uiuc.jdbv.diagram.Diagram;

/**
 * An abstract action for save-diagram related actions
 *
 * @version $Revision: 1.5 $
 * @author Brian Sidharta
 */
public abstract class AbstractSaveAction extends AbstractAction 
{
    /** Description for dbv files */
    public static final String DIAGRAM_FILE_DESC = "JDBV Save diagram files";

    /** The filename extension for Diagrams = "dbv" */
    public static final String[] DIAGRAM_EXTENSIONS = {"dbv"};

    /** Suffix that filenames must end in */
    public static final String FILENAME_SUFFIX = ".dbv";

    protected static JFileChooser fileChooser;

    /** The JDBV application instance that created this action */
    protected JDBV jdbv;

    public AbstractSaveAction( String name, JDBV jdbv )
    {
        super( name );
        this.jdbv = jdbv;
    }

    static {
	SuffixFileFilter fileFilter =
		 new SuffixFileFilter( DIAGRAM_FILE_DESC, DIAGRAM_EXTENSIONS );

	fileChooser = new ConfirmingFileChooser( "." );
	fileChooser.addChoosableFileFilter( fileFilter );

    }
    
    /** 
     * Shows a save-diagram dialog box
     * @return the filename the user selected or null if the user cancelled
     */
    protected String showSaveDialog()
    {
        int retval = fileChooser.showSaveDialog( jdbv.MAIN_WINDOW );
        if (retval == JFileChooser.APPROVE_OPTION) {
            String filename = fileChooser.getSelectedFile().getPath();
            if (!filename.toLowerCase().endsWith( FILENAME_SUFFIX )) {
                filename += FILENAME_SUFFIX;
            } // end of if ()
            
            return filename;

        }
        return null;
    }


    /**
     * Saves the supplied diagram to the filename it has set
     */
    protected void saveDiagram( Diagram diagram ) throws IOException
    {
        String filename = diagram.getFilename();
        XMLEncoder enc = 
            new JDBVXMLEncoder(new BufferedOutputStream
                               (new FileOutputStream(filename)));
        enc.writeObject( diagram );
        enc.close();
        diagram.setModified( false );
    }


    /**
     * JFileChooser that confirms overwriting existing files
     */
    protected static class ConfirmingFileChooser extends JFileChooser
    {
        public ConfirmingFileChooser( String currentDirectoryPath )
        {
            super( currentDirectoryPath );
        }


        /**
         * Called by UI when user hits Approve button.
         * Checks and confirms overwriting if selected file exists.
         */
        public void approveSelection()
        {
            File file = getSelectedFile();

            if (file.exists()) {
                StringBuffer sb = new StringBuffer();
                sb.append( "File " );
                sb.append( file.getPath());
                sb.append( " already exists.\n\nReplace it?" );
                int overwrite = JOptionPane.
                    showConfirmDialog( this, sb, "Save As",
                                       JOptionPane.OK_CANCEL_OPTION);

                // if user doesn't want to overwrite, we keep the dialog up
                if (overwrite == JOptionPane.CANCEL_OPTION) {
                    return;
                } // end of if ()
            } // end of if ()
            
            // Pass control to superclass to use the selected file
            super.approveSelection();
        }        
    }
    
}

