////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import edu.uiuc.jdbv.diagram.Diagram;

/**
 * Defines the action for closing the current diagram
 *
 * @author Brian Sidharta
 * @version $Revision: 1.7 $
 */
class CloseDiagramAction extends AbstractAction implements ChangeListener
{
    /** The logger to be used for the file = jdbv.CloseDiagramAction */
    private static final org.apache.log4j.Logger LOG = 
        org.apache.log4j.Logger.getLogger("jdbv.CloseDiagramAction");


    /** The short descriptive text of this action = "Close" */
    public static final String NAME_VALUE = "Close";

    /** The mnemonic keycode = KeyEvent.VK_I */
    public static final int MNEMONIC_CODE = KeyEvent.VK_C;

    /** The keystroke key-code = java.awt.event.KeyEevnt.VK_Z */
    public static final int KEY_STROKE_CODE = KeyEvent.VK_F4;

    /** The keystroke modifier mask = Event.CTRL_MASK */
    public static final int KEY_STROKE_MODIFIERS = Event.CTRL_MASK;

    /** The icon for the toolbar */
    public static final Icon SMALL_ICON_VALUE = 
        new ImageIcon( ClassLoader.getSystemResource
		       ("resources/icons/blank16.gif" ));


    /** The long description text of this action */
    public static final String LONG_DESCRIPTION_VALUE = 
        "Closes the current diagram.";

    /** Short description for tool-tip */
    public static final String SHORT_DESCRIPTION_VALUE = "Close";


    /** The JDBV application instance that created this ImportSchemaAction */
    private JDBV jdbv;

    private Diagram diagram = null;


    /** Constructor that takes the JDBV application instance */
    public CloseDiagramAction( JDBV jdbv )
    {
        super( NAME_VALUE );

        // Setup the action parameters
        KeyStroke keyStroke = 
            KeyStroke.getKeyStroke( KEY_STROKE_CODE, KEY_STROKE_MODIFIERS );
        putValue( ACCELERATOR_KEY, keyStroke );
        putValue( MNEMONIC_KEY, new Integer( MNEMONIC_CODE ));
        putValue( LONG_DESCRIPTION, LONG_DESCRIPTION_VALUE );
        putValue( SMALL_ICON, SMALL_ICON_VALUE );
        putValue( SHORT_DESCRIPTION, SHORT_DESCRIPTION_VALUE );

        this.jdbv = jdbv;
        updateState();
    }

    /** Handles the initiation of this action */
    public void actionPerformed( ActionEvent e )
    {
        initiate();
    }

    /**
     * Initiates the process of closes the current diagram
     * @return whether the operation completed successfully
     */
    public boolean initiate()
    {
        if (diagram != null) {
            if (diagram.isModified()) {
                int choice = showDialog();
                switch (choice) {
                case JOptionPane.YES_OPTION :
		    boolean isSaved = jdbv.SAVE_DIAGRAM_ACTION.initiate();
                    if (!isSaved) {
                        // Save failed, don't close diagram
                        return (false);
                    }
                    // Pass through to NO_OPTION
                case JOptionPane.NO_OPTION :
                    closeDiagram();
                    return (true);
                case JOptionPane.CANCEL_OPTION :
                    return (false);
                default:
                    LOG.error( "Unexpected value from showDialog(): " +
                               choice );
                    return (false);
                } // end of switch ()
            } else {
                closeDiagram();
                return (true);
            } // end of else
        }

        return (false);
    }

    private void closeDiagram()
    {
        jdbv.MAIN_WINDOW.removeDiagram( diagram );

    }


    private int showDialog()
    {
        StringBuffer sb = new StringBuffer();
        sb.append( "The \"" );
        sb.append( diagram.getName());
        sb.append( "\" diagram has changed.\n\n" );
        sb.append( "Save changes?" );

        return JOptionPane.showConfirmDialog( jdbv.MAIN_WINDOW,
                                              sb.toString(),
                                              "dbViZ - Close Diagram",
                                              JOptionPane.YES_NO_CANCEL_OPTION,
                                              JOptionPane.WARNING_MESSAGE);

    }


    /** for ChangeListener */
    public void stateChanged(ChangeEvent e) {
        updateState();
    }

    /** ChangeListener: updates enabled state */
    public void updateState()
    {
        diagram = jdbv.getCurrentDiagram();
	setEnabled( diagram != null );
    }
}

