////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;

/**
 * Defines the action for exiting the application
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.7 $
 */
class HelpAboutAction extends AbstractAction
{
    /** The name of the file that contains the info text */
    public static final String ABOUT_FILENAME = "resources/about.txt";

    /** The short descriptive text of this action = "HelpAbout" */
    public static final String NAME_VALUE = "About...";

    /** The mnemonic keycode = KeyEvent.VK_X */
    public static final int MNEMONIC_CODE = KeyEvent.VK_A;

    /** The long description text of this action */
    public static final String LONG_DESCRIPTION_VALUE = 
        "Displays information about the dbViZ application.";

    /** The icon for the toolbar */
    public static final Icon SMALL_ICON_VALUE = 
        new ImageIcon( "resources/icons/about16.gif" );

    /** Short description for tool-tip */
    public static final String SHORT_DESCRIPTION_VALUE = "About dbViZ";



    /** The buffer size */
    private static int BUFFER_SIZE = 1024;

    /** Other help text to dispaly */
    public static final String INFO_TEXT;

    /** The GUI component to show in the message box */
    private static final Component INFO_COMPONENT;

    /** The maximum width for the info text area */
    private static final int MAX_INFOTEXT_WIDTH = 480;

    /** The maximum height for the info text area */
    private static final int MAX_INFOTEXT_HEIGHT = 300;



    /** The JDBV application instance that created this HelpAboutAction */
    private JDBV jdbv;


    /** Static initializer - Load the info text */
    static {
        URL aboutURL = ClassLoader.getSystemResource( ABOUT_FILENAME );
        String info;
        
        if (aboutURL == null) {
            info = "Error: no file found: " + ABOUT_FILENAME;
        } else {
            try {
                StringWriter stringOut = new StringWriter( BUFFER_SIZE );
                Reader fileIn = new InputStreamReader( aboutURL.openStream() );
                char[] buffer = new char[BUFFER_SIZE];
                int charsRead;
                do {
                    charsRead = fileIn.read( buffer );
                    stringOut.write( buffer, 0, charsRead );
                } while (charsRead == BUFFER_SIZE);
                info = stringOut.toString();
            } catch (IOException ioE) {
                info = "Error: couldn't read file: " + ABOUT_FILENAME +
                    "\nMessage: " + ioE.getMessage();
            } // end of try-catch
        } // end of else

        INFO_TEXT = info;
        INFO_COMPONENT = createInfoComponent();
    }

    /**
     * Creates the info-message GUI component */
    private static JComponent createInfoComponent()
    {
        JPanel panel = new JPanel( new BorderLayout());
        JLabel generalLabel = new JLabel( "Version: " + JDBV.VERSION);
        JTextArea infoArea = new JTextArea( INFO_TEXT );
        infoArea.setEditable( false );
        JScrollPane infoScroller = new JScrollPane( infoArea );
        infoScroller.setVerticalScrollBarPolicy
            ( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        Dimension dim = infoScroller.getPreferredSize();
        dim.width = Math.min( MAX_INFOTEXT_WIDTH, dim.width );
        dim.height = Math.min( MAX_INFOTEXT_HEIGHT, dim.height );
        infoScroller.setPreferredSize( dim );
        
        panel.add( generalLabel, BorderLayout.NORTH );
        panel.add( infoScroller, BorderLayout.CENTER );

        return (panel);
    }


    /** Constructor that takes the JDBV application instance */
    public HelpAboutAction( JDBV jdbv )
    {
        super( NAME_VALUE );

        // Setup the action parameters
        putValue( MNEMONIC_KEY, new Integer( MNEMONIC_CODE ));
        putValue( LONG_DESCRIPTION, LONG_DESCRIPTION_VALUE );
        putValue( SMALL_ICON, SMALL_ICON_VALUE );
        putValue( SHORT_DESCRIPTION, SHORT_DESCRIPTION_VALUE );

        this.jdbv = jdbv;
        
    }

    /** Handles the initiation of this action */
    public void actionPerformed( ActionEvent e )
    {
        JOptionPane.showMessageDialog( jdbv.MAIN_WINDOW, INFO_COMPONENT,
                                       "About " + JDBV.PROGRAM_NAME,
                                       JOptionPane.INFORMATION_MESSAGE );
    }

}

