////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

import edu.uiuc.jdbv.schema.Column;

import java.util.List;
import java.util.ArrayList;

import java.awt.*;
import java.awt.geom.*;
import java.io.Serializable;

import com.jgraph.JGraph;
import com.jgraph.graph.*;



public class ConnectionEdgeRenderer extends EdgeRenderer
    implements CellViewRenderer, Serializable 
{
    /** horizontal distance to place dep col mergepoint from table edge */
    public static final int MERGEPOINT_DISTANCE = 16;

    /** The pixel radius of the mergepoint */
    public static final int MERGEPOINT_RADIUS = 4;

    /** The pixel radius of the dependent column nubs */
    public static final int NUB_RADIUS = 4;


    /** The shape for the merge point */
    protected transient Ellipse2D mergePointShape = null;

    /** Cached dependent table view */
    protected transient TableView depTableView = null;

    /** Cached dependent columns */
    protected transient Column[] depCols = null;

    /** List of ShapeS of dependent column lines */    
    protected transient List depColsShapeList = new ArrayList( 2 );

    /** list of ShapeS of depCol nub shapes */
    protected transient List nubsShapeList = new ArrayList( 2 );

    /**
     * Constructs a renderer that may be used to render edges.
     */
    public ConnectionEdgeRenderer() {
    }


    /**
     * Configure and return the renderer based on the passed in
     * components. The value is typically set from messaging the
     * graph with <code>convertValueToString</code>.
     *
     * @param   graph the graph that that defines the rendering context.
     * @param   value the object that should be rendered.
     * @param   selected whether the object is selected.
     * @param   hasFocus whether the object has the focus.
     * @param   isPreview whether we are drawing a preview.
     * @return	the component used to render the value.
     */
    public Component getRendererComponent( JGraph graph, CellView view,
                                           boolean sel, boolean focus,
                                           boolean preview)
    {
        ConnectionEdgeView connView = (ConnectionEdgeView)view;
        ConnectionEdge connEdge = (ConnectionEdge)connView.getCell();
        depTableView = connView.getDependentTableView();
        depCols = connEdge.getConnection().getDependantColumns();
        return (super.getRendererComponent( graph, view, sel, focus, preview));
    }


        

    /**
     * Returns the bounds of the edge shape.
     */
    public Rectangle getBounds(CellView value) {
        if (value instanceof EdgeView && value != null) {
            view = (EdgeView) value;
            installAttributes(view);
            Rectangle r = getPaintBounds(view);
            Rectangle label = getLabelBounds(view);
            if (label != null)
                r = r.union(label);
            int b = (int) Math.ceil(lineWidth);
            r.x = r.x - b;
            r.y = r.y - b;
            r.width = r.width + 2 * b;
            r.height = r.height + 2 * b;
            return r;
        }
        return null;
    }


    /**
     * Returns the bounds of the edge shape without label
     */
    public Rectangle getPaintBounds(EdgeView view) {
        this.view = view;
        installAttributes(view);
        return createShape().getBounds();
    }


    /**
     * Paint the renderer.
     */
    public void paint(Graphics g) {
        ConnectionEdgeView connView = (ConnectionEdgeView)view;

        super.paint( g );


        // Draw our dependent column stuff
        Graphics2D g2 = (Graphics2D) g;
        int c = BasicStroke.CAP_BUTT;
        int j = BasicStroke.JOIN_MITER;
        g2.setStroke(new BasicStroke(lineWidth, c, j));

        // This is a hack because clip bounds werent being set right
        // by JGraph
        g.setClip( null );

        if (connView.areAllColsVisible()) {
            // Draw depcols lines and mergepoint
            g2.setColor(getForeground());
            g2.fill( mergePointShape );
            for (int i = depColsShapeList.size() - 1; i >= 0; --i) {
                Shape line = (Shape)depColsShapeList.get( i );
                Shape nub = (Shape)nubsShapeList.get( i );
                g2.draw( line );
                g2.fill( nub );
            } // end of for ()
            
        } else {
            g2.setColor( getForeground());

            // Draw depcols lines and nubs
            for (int i = depColsShapeList.size() - 1; i >= 0; --i) {
                Shape line = (Shape)depColsShapeList.get( i );
                Shape nub = (Shape)nubsShapeList.get( i );
                g2.draw( line );
                g2.fill( nub );
            } // end of for ()

            // Draw mergepoint background fill
            g2.setColor( getBackground());
            g2.fill( mergePointShape );

            // Draw mergepoint border
            g2.setColor( getForeground());
            g2.draw( mergePointShape );
        } // end of else

    }


    /**
     * Returns the shape that represents the current edge
     * in the context of the current graph.
     * This method sets the global beginShape, lineShape
     * and endShape variables as a side-effect.
     */
    protected Shape createShape() {
        super.createShape();

        ConnectionEdgeView connView = (ConnectionEdgeView)view;

        // Create mergepoint shape
        Point mergePoint = view.getPoint( 0 );
        mergePointShape = 
            new Ellipse2D.Float( mergePoint.x - MERGEPOINT_RADIUS,
                                 mergePoint.y - MERGEPOINT_RADIUS,
                                 MERGEPOINT_RADIUS * 2,
                                 MERGEPOINT_RADIUS * 2 );
        sharedPath.append( mergePointShape, false );
        
        // Fast abort if the edge points haven't been initialized yet
        if (depCols != null && depTableView != null) {
            // Create dep-column-connector line shapes
            depColsShapeList.clear();
            nubsShapeList.clear();

            int tableX = depTableView.getCenterPoint().x;

            for (int i = depCols.length - 1; i >= 0; --i) {
                Column col = depCols[i];

                // if the column's hidden, don't draw a line
                if( depTableView.getYOffsetForColumn( col ) != 
                    TableView.COLUMN_HIDDEN_OFFSET ) {
                    createDepColShapes( col, mergePoint, tableX );
                } // end of if ()
            } // end of for ()

            // if all columns are hidden, draw line to heading
            if (depColsShapeList.size() == 0) {
                createDepColShapes( depCols[0], mergePoint, tableX );
            } // end of if ()
            
        } // end of else
        
        return (sharedPath);
    }


    protected void createDepColShapes( Column col, Point mergePoint, 
                                       int tableX )
    {
        Point colPoint = depTableView.getColumnPoint( col, mergePoint);
        Line2D line = new Line2D.Float( mergePoint, colPoint );
        Arc2D nub;
        float extentAngle;
        
        depColsShapeList.add( line );
        sharedPath.append( line, true );
        
        if (colPoint.x < tableX) { // Make column nub on left
            extentAngle = 180.0f;
        } else { // Make column nub on right
            extentAngle = -180.0f;
        }
        nub = new Arc2D.Float( colPoint.x - NUB_RADIUS,
                               colPoint.y - NUB_RADIUS,
                               NUB_RADIUS * 2, NUB_RADIUS * 2,
                               90.0f, extentAngle, Arc2D.PIE );

        nubsShapeList.add( nub );
        sharedPath.append( nub, true );
        

    }



    /**
     * Returns <code>true</code> if <code>key</code> is a supported
     * attribute in the renderer. Supported attributes affect the
     * visual appearance of the renderer.
     *
     * @param   key the key that defines the attribute to be checked.
     * @return	true if <code>key</code> is supported by this renderer.
     */
    public boolean supportsAttribute(Object key) {
        return key.equals(GraphConstants.POINTS)
            || key.equals(GraphConstants.BACKGROUND)
            || key.equals(GraphConstants.BORDER)
            || key.equals(GraphConstants.FONTNAME)
            || key.equals(GraphConstants.FONTSIZE)
            || key.equals(GraphConstants.FONTSTYLE)
            || key.equals(GraphConstants.FOREGROUND)
            || key.equals(GraphConstants.BORDERCOLOR)
            || key.equals(GraphConstants.LABELPOSITION)
            || key.equals(GraphConstants.LINESTYLE)
            || key.equals(GraphConstants.LINECOLOR)
            || key.equals(GraphConstants.LINEWIDTH)
            || key.equals(GraphConstants.DASHPATTERN)
            || key.equals(GraphConstants.LINEBEGIN)
            || key.equals(GraphConstants.BEGINSIZE)
            || key.equals(GraphConstants.BEGINFILL)
            || key.equals(GraphConstants.LINEEND)
            || key.equals(GraphConstants.ENDSIZE)
            || key.equals(GraphConstants.ENDFILL)
            || key.equals(GraphConstants.OPAQUE);
    }

}
