////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

//import edu.uiuc.jdbv.diagram.LayoutAlgorithm;

import com.jgraph.JGraph;
import com.jgraph.graph.*;
import com.jgraph.event.GraphModelListener;
import com.jgraph.event.GraphModelEvent;

import java.util.List;
import java.util.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.undo.*;
import java.beans.*;

import edu.uiuc.jdbv.JDBV;
import edu.uiuc.jdbv.schema.*;


/**
 * A diagram component that displays tables and the links between them
 * in a database schema. This is the model of MVC.
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.28 $
 */
public class Diagram extends JGraph implements GraphModelListener, Observer
{
    /** Property name for getAutoArrangeMode() */
    public static final String AUTOARRANGEMODE_PROPERTY = "AutoArrangeMode";

    /** Property name for isModified() */
    public static final String MODIFIED_PROPERTY = "Modified";

    /** Property name for filename */
    public static final String FILENAME_PROPERTY = "Filename";


    /** delimiter for directories in file paths */
    private static final char FILEPATH_SEPARATOR_CHAR = 
        java.io.File.separatorChar;

    /** The logger to be used for the file */
    private static final org.apache.log4j.Logger LOG = 
        org.apache.log4j.Logger.getLogger("jdbv.diagram.Diagram");



    /** The prefix name for new unnamed diagrams */
    private static final String UNNAMED_DIAGRAM_NAME_PREFIX =
        "unnamed_";


    /** The initial capacity for the table hashset (should be prime) */
    private static final int INITIAL_TABLE_CAPACITY = 31;


    /** The default table border color for new diagrams */
    public static final Color DEFAULT_TABLE_BORDER_COLOR =
        Color.black;

    /** The default table foreground color for new diagrams */
    public static final Color DEFAULT_TABLE_FOREGROUND_COLOR =
        Color.black;

    /** The default table background color for new diagrams */
    public static final Color DEFAULT_TABLE_BACKGROUND_COLOR =
        new Color( 0.8f, 0.9f, 0.95f );

    /** The default grid color for diagrams */
    public static final Color DEFAULT_GRID_COLOR =
        new Color( 0.65f, 0.65f, 0.65f );

    /** The default grid size */
    public static final int DEFAULT_GRID_SIZE = 10;

    /** Default table height */
    public static final int DEFAULT_TABLE_HEIGHT = 120;
    /** Default table width */
    public static final int DEFAULT_TABLE_WIDTH = 100;

    /** The number of untitled (new) diagrams created */
    private static int untitledCount = 0;


    /** Each diagram has it's own Undo Manager */
    private final GraphUndoManager UNDO_MANAGER;
	
    /** The schema for this diagram */
    private Schema schema;

    /** Maps Table to TableGraphCells */
    private Map tableCellMap;

    /** Maps Table to CachedPropreties */
    private Map tableCacheMap;

    /** The attribute map to give to new Views */
    private Map defaultViewAttributeMap;

    /** offset from corner to stick next table */
    private int nextLocation = 10;

    /**
     * determines if tables are rearranged whenever a table
     * or removed from a diagram 
     */
    private boolean autoArrangeMode = false;

    /** Whether this diagram has been modified since its last save */
    private boolean modified = false;

    /** The diagram's temporary name if it's new */
    private String tempName = null;


    /** Constructor that takes the Schema that this diagram is for */
    public Diagram( Schema schema )
    {
        this( null, schema );
    }


    /** Constructor that takes the name of the diagram */
    public Diagram( String name, Schema schema )
    {
        super( new DefaultGraphModel());

        if (name == null) {
            tempName = UNNAMED_DIAGRAM_NAME_PREFIX + (untitledCount++);
        } // end of if ()
      
        setName( name );
        setupDefaultSettings();


        if( schema == null ) {
            throw new NullPointerException( "Schema is null!" );
        }
        this.schema = schema;

        tableCellMap = new HashMap( INITIAL_TABLE_CAPACITY );
        tableCacheMap = new HashMap( INITIAL_TABLE_CAPACITY );

	UNDO_MANAGER = new GraphUndoManager();

        setupListeners();
    }


    /**
     * Sets up the default JGraph settings
     */
    protected void setupDefaultSettings()
    {
        setGridColor( DEFAULT_GRID_COLOR );
        setGridSize( DEFAULT_GRID_SIZE );
        setAntiAliased( false );
        setBendable( true );
        setCloneable( false );
        setConnectable( false );
        setDisconnectable( false );
        setDisconnectOnMove( false );
        setDragEnabled( false );
        setDropEnabled( false );
        setEditable( false );
        setGridEnabled( true );
        setGridVisible( true );
        setMoveable( true );
        setSizeable( true );
        setModified( true );

        defaultViewAttributeMap = createDefaultAttributeMap();
    }


    /**
     * Sets up the event listeners for this diagram
     */
    protected void setupListeners()
    {
	getModel().addUndoableEditListener(JDBV.getInstance()); 

        // Setup listeners to update the modified state
        getModel().addGraphModelListener(this);
        getView().addObserver(this);

    }



    /**
     * Adds a table to the diagram
     * @param table the table to add, which must be in the current schema
     * @return whether the table was added successfully or not
     */
    public boolean addTable( Table table )
    {
        Map modelAttrmap = null;
        Map viewAttrmap = null;

        //checks to see if table has been added before
        if( tableCacheMap.containsKey( table ))
        {
            CachedProperties props = (CachedProperties)
                tableCacheMap.get( table );
            modelAttrmap = props.modelAttrmap;
            viewAttrmap = props.viewAttrmap;
            addTable( table, modelAttrmap, viewAttrmap );
        }
        else
        {
            int offset = 10;
            Point location = new Point( nextLocation, nextLocation );
            addTable( table, location );
            if( nextLocation < 120 ) {
                nextLocation += offset;
            } else {
                nextLocation = offset;
            }

            if (autoArrangeMode) {
                autoArrangeTables();
            }
        }
        return (true);
    }

    /**
     * Adds a table to the diagram at the specified location
     * @param table the table to add, which must be in the current schema
     * @param location the x,y location on the diagram where the center
     *        of the table should be displayed
     */
    public void addTable( Table table, Point location )
    {  
        //Check to see if table has been cached
        Map modelAttrmap = null;
        Map viewAttrmap = null;

        // Add to JGraph
        Rectangle bounds = new Rectangle( location.x, location.y, 
                                          DEFAULT_TABLE_HEIGHT, 
                                          DEFAULT_TABLE_WIDTH );
        // Create Attribute Map for Cell
        modelAttrmap = GraphConstants.createMap();
        viewAttrmap = GraphConstants.cloneMap( defaultViewAttributeMap );
        GraphConstants.setBounds( viewAttrmap, bounds );
        
        addTable( table, modelAttrmap, viewAttrmap );
    }


    /**
     * Adds a table to the diagram using the supplied attribute maps
     * @param table the table to add, which must be in the current schema
     * @param modelAttributeMap the attributeMap to pass to the TableGraphCell
     * @param viewAttributeMap the attributeMap to pass to the table Views
     */
    public void addTable( Table table, Map modelAttributeMap, 
                          Map viewAttributeMap )
    {
        // Map that Contains Attribute Maps
        Map cellAttrmapMapMap = new HashMap(3);

        // Create Vertex
        TableGraphCell cell = new TableGraphCell( table, modelAttributeMap );
        // Associate Attribute Map with Cell
        cellAttrmapMapMap.put(cell, viewAttributeMap);
        // Create Default Floating Port
        DefaultPort port = new DefaultPort("Floating");
        cell.add(port);

        tableCellMap.put( table, cell );

        // Add Cell (and Children) to the Model
        List insertList = new ArrayList( 3 );
        insertList.add( cell );
        ConnectionSet cs = new ConnectionSet();

        // Determine if there are any links to draw
        Connection[] connections;
        connections = table.getStartingConnections();
        for( int i = 0; i < connections.length; i++ ) {
            Connection conn = connections[i];
            Table otherTable = conn.getIndependantTable();
            if( containsTable( otherTable )) {
                addConnection( conn, cs, insertList );
            }
        }
        connections = table.getEndingConnections();
        for( int i = 0; i < connections.length; i++ ) {
            Connection conn = connections[i];
            Table otherTable = conn.getDependantTable();
            if( containsTable( otherTable )) {
                addConnection( conn, cs, insertList );
            }
        }

        Object[] inserts = insertList.toArray();
        getModel().insert( inserts, cs, null, cellAttrmapMapMap);
    }


    /**
     * Appends diagram insertion data to data structures for creating
     * a Connection edge between two tables that are on the
     * diagram already.
     * @param conn the Connection that should have an edge added
     * @param cs a ConnectionSet that will be inserted into the graph
     * @param insertList a List of graph cells that will be inserted into
     *        the graph
     */
    private void addConnection( Connection conn, ConnectionSet cs,
                                List insertList )
    {
        TableGraphCell independantCell =
            (TableGraphCell)getCellForTable( conn.getIndependantTable());
        TableGraphCell dependantCell =
            (TableGraphCell)getCellForTable( conn.getDependantTable());
        DefaultEdge edge = new ConnectionEdge( conn );
        Map edgeAttribs = GraphConstants.createMap();
        GraphConstants.setLineEnd( edgeAttribs, GraphConstants.CLASSIC );
        edge.setAttributes( edgeAttribs );

        cs.connect( edge, getDefaultPort( dependantCell ), 
                    getDefaultPort( independantCell));
        insertList.add( edge );
    }


    private void autoArrangeTables() {
        LOG.debug("Rearranging Tables!"); 	
        // set the spacing relative to the table size, to look nice
        LayoutAlgorithm alg = new LayoutAlgorithm((JGraph) this,
                                (int) (2.5 * DEFAULT_TABLE_WIDTH),
                                (int) (1.5 * DEFAULT_TABLE_HEIGHT));
        alg.run(true);
        updateUI();
    }
	

    /**
     * Gets the first port in the vertex
     */
    private Port getDefaultPort( Object vertex )
    {
        GraphModel model = getModel();
        int numChildren = model.getChildCount( vertex );

        for( int i = 0; i < numChildren; i++ ) {
            Object child = model.getChild( vertex, i );
            if( child instanceof Port ) {
                return ((Port)child);
            }
        }
        return (null); // No ports found
    }


    /**
     * Removes a table from the diagram
     * @param table the table to be removed, which must be in the diagram
     */
    public void removeTable( Table table )
    {
        DefaultGraphCell graphCell = (DefaultGraphCell)
            tableCellMap.remove( table );
        //store info 
        Map viewAttrmap = getView().getMapping( graphCell, true )
            .getAttributes();
        Map modelAttrmap = graphCell.getAttributes();
        tableCacheMap.put( table, new CachedProperties( modelAttrmap,
                                                        viewAttrmap ));

        ArrayList removeList = new ArrayList( 3 );
        
        removeList.add( graphCell );
        // Remove all edges connected to the port
        Port defaultPort = getDefaultPort( graphCell );
        removeList.add( defaultPort );
        for( Iterator edgeIt = defaultPort.edges(); edgeIt.hasNext(); ) 
            {
                removeList.add( edgeIt.next());
            }
        getModel().remove( removeList.toArray() );
    }


    /**
     * Returns whether a table is contained in this diagram
     * @return <code>true</code> if the table is in this diagram, or 
     *         false otherwise.
     */
    public boolean containsTable( Table table )
    {
        return (tableCellMap.containsKey( table ));
    }



    /**
     * Returns the GraphCell for the supplied table.
     * @param table a table currently in this diagram
     * @return the TableGraphCell for the specified table
     */
    public Object getCellForTable( Table table )
    {
        return (tableCellMap.get( table ));
    }


    /**
     * Returns the number of tables in this diagram.
     */
    public int getNumTables()
    {
        return (tableCellMap.size());
    }

    /**
     * Returns the presentation name of the diagram
     */
    public String getName()
    {
        String superName = super.getName();
        
        if (superName == null) {
            return (tempName);
        } else {
            // Return name without path
            int index = superName.lastIndexOf( FILEPATH_SEPARATOR_CHAR );
            if (index >= 0) {
                return (superName.substring( index + 1 ));
            } else {
                return (superName);                 
            } // end of else
        } // end of else
    }


    /**
     * Returns the filename for the diagram, or null if the diagram hasn't
     * been saved before.
     */
    public String getFilename()
    {
        return (super.getName());
    }


    /**
     * Sets the filename for the diagram
     */
    public void setFilename( String filename )
    {
        String oldName = getName();
        setName( filename );
        if (!filename.equals( oldName )) {
            firePropertyChange( FILENAME_PROPERTY, oldName, filename );
        } // end of if ()

    }


    /**
     * Retrieves all cells in the graph
     */
    public Object[] getAll() 
    {
        return getDescendants( getRoots());
    }


    /**
     * Retrieves all diagram data in this graph.
     * @return an object that can be given to setAllData() to recreate the
     *         Diagram
     */
    public Object getGraphData()
    {
        Object[] allCells = getAll();
        Map viewAttrmapMap = 
            GraphConstants.createPropertyMap( allCells, getView());
        ArrayList dataList = new ArrayList(4);
        dataList.add( new Integer( nextLocation ));  // slot 0
        dataList.add( allCells );  // slot 1
        dataList.add( tableCellMap ); // slot 2
        dataList.add( viewAttrmapMap );  // slot 3

        return (dataList);
    }


    /**
     * Sets all diagram data in this graph. A side effect is that it clears
     * the undo stack so an "Open Diagram" command can't be undone.
     * @param graphData an object created by getGraphData()
     */
    public void setGraphData( Object graphData )
    {
        if (graphData instanceof ArrayList) {
            ArrayList dataList = (ArrayList)graphData;
            nextLocation = ((Integer)dataList.get( 0 )).intValue();
            Object[] allCells = (Object[])dataList.get( 1 );
            tableCellMap = (Map)dataList.get( 2 );
            Map viewAttrmapMap = (Map)dataList.get( 3 );
            
            getModel().insert( allCells, null, null, viewAttrmapMap );

            // Clear the undo manager so we can't undo the load
            UNDO_MANAGER.discardAllEdits();
        } // end of if ()
        
    }


    /**
     * Retrieves the schema that this diagram is for
     */
    public Schema getSchema()
    {
        return (schema);
    }

    /**
     * Retruns the UNDO_MANAGER to be used with this diagram
     */
    public UndoManager getUndoManager()
    {
	return UNDO_MANAGER;
    }

    /** Returns whether the diagram is auto-arranging cells */
    public boolean getAutoArrangeMode() {
        return(autoArrangeMode);
    }

    /** Sets whether the diagram is auto-arranging cells */
    public void setAutoArrangeMode (boolean value) {
        if (autoArrangeMode != value) {
            boolean oldValue = autoArrangeMode;
            autoArrangeMode = value;
            firePropertyChange( MODIFIED_PROPERTY, oldValue, value );
        } // end of if ()

        if (autoArrangeMode) {
            // rearrange tables;
            autoArrangeTables();
            setMinimumMove( Integer.MAX_VALUE );
            JDBV.getInstance().MAIN_WINDOW.repaint();
        } else {
            setMinimumMove( 0 );
        } // end of else
        

    }


    /**
     * Returns Whether this diagram has been modified since its last save
     * @return value of modified.
     */
    public boolean isModified() {
        return modified;
    }
    
    /**
     * Set Whether this diagram has been modified since its last save
     * @param v  Value to assign to modified.
     */
    public void setModified(boolean  v) {
        if (v != modified) {
            boolean oldValue = this.modified;
            this.modified = v;
            firePropertyChange( MODIFIED_PROPERTY, oldValue, v );
        } // end of if ()
        
    }


    /** for View Observer */
    public void update(Observable obs, Object arg) {
        setModified( true );
    }

    /** for GraphModelListener */
    public void graphChanged(GraphModelEvent e) {
        setModified( true );
    }

    /** Sets the default attribute Map to give to new cell Views */
    public void setDefaultViewAttributes( Map attrMap )
    {
        defaultViewAttributeMap = attrMap;
    }

    /** Gets the default attribute map that's given to new cell views */
    public Map getDefaultViewAttributes()
    {
        return (defaultViewAttributeMap);
    }


    protected VertexView createVertexView( Object v, CellMapper cm )
    {
        VertexView view;
        if( v instanceof TableGraphCell ) {
            view = new TableView( (TableGraphCell)v, this, cm );
        } else {
            view = super.createVertexView( v, cm );
        }

        return (view);
    }


    protected EdgeView createEdgeView(Edge e, CellMapper cm) 
    {
        if (e instanceof ConnectionEdge) {
            return (new ConnectionEdgeView( e, this, cm ));
        } else {
            return (super.createEdgeView( e, cm ));
        } // end of else
    }


    protected PortView createPortView(Port p, CellMapper cm)
    {
        if (p instanceof DefaultPort) {
            DefaultPort defPort = (DefaultPort)p;
            if (defPort.getParent() instanceof TableGraphCell) {
                return (new TablePortView( p, this, cm ));
            } // end of if ()
        }

        return (super.createPortView( p, cm ));
    }
        

    /**
     * Creates a default attribute map for cell views
     */
    protected static Map createDefaultAttributeMap()
    {
        Map attrMap = GraphConstants.createMap();
        GraphConstants.setOpaque( attrMap, true );
        GraphConstants.setBorderColor( attrMap, DEFAULT_TABLE_BORDER_COLOR );
        GraphConstants.setForeground( attrMap, DEFAULT_TABLE_FOREGROUND_COLOR);
        GraphConstants.setBackground( attrMap, DEFAULT_TABLE_BACKGROUND_COLOR);
        return (attrMap);
    }

    
    private class CachedProperties
    {
        public Map modelAttrmap;
        public Map viewAttrmap;

        public CachedProperties( Map modelAttrmap, Map viewAttrmap )
        {
            this.modelAttrmap = modelAttrmap;
            this.viewAttrmap = viewAttrmap;
        }
    }


}
