////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

import com.jgraph.JGraph;
import com.jgraph.graph.*;

import java.awt.Point;
import java.awt.Rectangle;

import java.util.*;

/**
 * Arranges the nodes with the Sugiyama Layout Algorithm.<br>
 *
 * <a href="http://plg.uwaterloo.ca/~itbowman/CS746G/Notes/Sugiyama1981_MVU/">
 *  Link to the algorithm</a>
 *
 *<br>
 *<br>
 * @author Aleksandra Faust <br>
 * @version $Revision: 1.4 $
 */
public class LayoutAlgorithm {

	/** Const to add Attributes at the Nodes
	 *
	 */
	public static final String VISITED = "Visited";

	/** Const to add the Cell Wrapper to the Nodes
	 */
	public static final String CELL_WRAPPER = "CellWrapper";
	
	/** Determines the spacing between the nodes
	 */
	public static int HORIZONTAL_SPACING = 150;
	public static int VERTICAL_SPACING = 150;

	/** represents the size of the grid in horizontal grid elements
	 *
	 */
	private int gridAreaSize = Integer.MIN_VALUE;
	
	private JGraph jgraph;
	private GraphModel model;
	/**
	 * Implementation.
	 *
	 * First of all the Algorithm searches the roots from the
	 * Graph. Starting from this roots the Algorithm creates
	 * levels and stores them in the member <code>levels</code>.
	 * The Member levels contains Vector Objects and the Vector per level
	 * contains Cell Wrapper Objects. After that the Algorithm
	 * tries to solve the edge crosses from level to level and
	 * goes top down and bottom up. After minimization of the
	 * edge crosses the algorithm moves each node to its
	 * bary center. Last but not Least the method draws the Graph.
	 *
	 * @see LayoutAlgorithm
	 *
	 */
	public LayoutAlgorithm () {
	}
	
	public LayoutAlgorithm (JGraph Jgraph, int HorizontalSpacing, int VerticalSpacing) {
		
		jgraph = Jgraph;
		model = jgraph.getModel();

		HORIZONTAL_SPACING = HorizontalSpacing;
		VERTICAL_SPACING = VerticalSpacing;
	}	
	public void run (boolean applyToAll) {		
		
		Object[] selectedCells =
			(applyToAll ? jgraph.getRoots() : jgraph.getSelectionCells());
		CellView[] selectedCellViews =
			jgraph.getView().getMapping(selectedCells); 

		// List to Store the Vertices
		List listVertices = new ArrayList();
		
		// Loop through all views and remove non vertexes
		for (int i = 0; i < selectedCellViews.length; i++) {
		  // Add Vertex to List
		  if (selectedCellViews[i] instanceof VertexView) {
			  listVertices.add(selectedCellViews[i]);
		  }
		}
		
		CellView[] vertices = new CellView[listVertices.size()]; 
		vertices = (CellView[])listVertices.toArray( vertices );
				
		// search all roots
		Vector roots = searchRoots(selectedCellViews);

		// return if no root found
		if (roots.size() == 0)
			return;

		// create levels
		Vector levels = fillLevels(selectedCellViews, roots);

        // solves the edge crosses
		solveEdgeCrosses(levels);

        // move all nodes into the barycenter
		moveToBarycenter(selectedCellViews, levels);
		
		Point min = findMinimumAndSpacing(vertices);

        // draw the graph in the window

		drawGraph(levels, min);
	}


	/** Searches all Roots for the current Graph
	 *  First the method marks any Node as not visited.
	 *  Than calls searchRoots(MyGraphCell) for each
	 *  not visited Cell.
	 *  The Roots are stored in the Vector named roots
	 *
	 * 	@return returns a Vector with the roots
	 *  @see #searchRoots(JGraph, CellView[])
	 */
	private Vector searchRoots(CellView[] selectedCellViews) {

		// get all cells and relations
		Vector vertexViews = new Vector(selectedCellViews.length);
		Vector roots = new Vector();

		// first: mark all as not visited
		// O(allCells&Edges)
		for (int i = 0; i < selectedCellViews.length; i++) {
			if (selectedCellViews[i] instanceof VertexView) {
				VertexView vertexView = (VertexView) selectedCellViews[i];
				vertexView.getAttributes().remove(VISITED);
				vertexViews.add(selectedCellViews[i]);
			}
		}

		// O(graphCells)
		for (int i = 0; i < vertexViews.size(); i++) {
			VertexView vertexView = (VertexView) vertexViews.get(i);
			if (vertexView.getAttributes().get(VISITED) == null) {
				searchRoots(vertexView, roots);
			}
		}

		return roots;
	}

	/** Searches Roots for the current Cell.
	 *
	 *  Therefore he looks at all Ports from the Cell.
	 *  At the Ports he looks for Edges.
	 *  At the Edges he looks for the Target.
	 *  If the Ports of the current Cell contains the target ReViewNodePort
	 *  he follows the edge to the source and looks at the
	 *  Cell for this source.
	 *
	 *  @param graphCell The current cell
	 */
	private void searchRoots(
		VertexView vertexViewToInspect,
		Vector roots) {
		// the node already visited
		if (vertexViewToInspect.getAttributes().get(VISITED)!= null) {
			return;
		}

		// mark as visited for cycle tests
		vertexViewToInspect.getAttributes().put(VISITED, new Boolean(true));


		// get all Ports and search the relations at the ports
		//List vertexPortViewList = new ArrayList() ;

		Object vertex = vertexViewToInspect.getCell();

		int portCount = model.getChildCount(vertex);
		for (int j = 0; j < portCount; j++) {
			Object port = model.getChild(vertex, j);
			
			// The current node is never a Target Node
			// -> The current node is a root node
			if (isRoot(port, roots)) {
				roots.add(vertexViewToInspect);
			}
		}
	}
	private boolean isRoot(Object port, Vector roots) {
		// Test all relations for where
		// the current node is a target node
		// for roots

		boolean isRoot = true;

		Iterator itrEdges = model.edges(port);
		while (itrEdges.hasNext()) {
			Object edge = itrEdges.next();

			// if the current node is a target node
			// get the source node and test
			// the source node for roots

			if (model.getTarget(edge) == port) {
				Object sourcePort = model.getSource(edge);

				Object sourceVertex = model.getParent(sourcePort);

				CellView sourceVertexView =
					jgraph.getView().getMapping(sourceVertex,false);
				if (sourceVertexView instanceof VertexView) {
					searchRoots((VertexView) sourceVertexView, roots);
					isRoot = false;
				}
			}		
		}
		return isRoot;
	}
	/** Method fills the levels and stores them in the member levels.

	 *  Each level was represended by a Vector with Cell Wrapper objects.
	 *  These Vectors are the elements in the <code>levels</code> Vector.
	 *
	 */
	private Vector fillLevels(
		CellView[] selectedCellViews,
		Vector rootVertexViews) {
		Vector levels = new Vector();

		// mark as not visited
		// O(allCells)
		for (int i = 0; i < selectedCellViews.length; i++) {
			CellView cellView = selectedCellViews[i];
			cellView.getAttributes().remove(VISITED);
		}

		Enumeration enumRoots = rootVertexViews.elements();
		while (enumRoots.hasMoreElements()) {
			VertexView vertexView = (VertexView) enumRoots.nextElement();
			fillLevels(levels, 0, vertexView);
		}

		return levels;

	}

	/** Fills the Vector for the specified level with a wrapper
	 *  for the MyGraphCell. After that the method called for
	 *  each neighbor graph cell.
	 *
	 *  @param level        The level for the graphCell
	 *  @param graphCell    The Graph Cell
	 */
	private void fillLevels(
		Vector levels,
		int level,
		VertexView vertexView) {
		// be sure that a Vector container exists for the current level
		if (levels.size() == level)
			levels.insertElementAt(new Vector(), level);

		// if the cell already visited return
		if (vertexView.getAttributes().get(VISITED) != null) {
			return;
		}

		// mark as visited for cycle tests
		vertexView.getAttributes().put(VISITED, new Boolean(true));

		// put the current node into the current level
		// get the Level Vector
		Vector vecForTheCurrentLevel = (Vector) levels.get(level);

		// Create a wrapper for the node
		int numberForTheEntry = vecForTheCurrentLevel.size();

		CellWrapper wrapper =
			new CellWrapper(level, numberForTheEntry, vertexView);

		// put the Wrapper in the LevelVector
		vecForTheCurrentLevel.add(wrapper);

		// concat the wrapper to the cell for an easy access
		vertexView.getAttributes().put(CELL_WRAPPER, wrapper);

		// if the Cell has no Ports we can return, there are no relations
		Object vertex = vertexView.getCell();
		
		int portCount = model.getChildCount(vertex);

		// iterate any NodePort
		for (int i = 0; i < portCount; i++) {

			Object port = model.getChild(vertex, i);

			// iterate any Edge in the port
			Iterator itrEdges = model.edges(port);

			while (itrEdges.hasNext()) {
				Object edge = itrEdges.next();

				// if the Edge is a forward edge we should follow this edge
				if (port == model.getSource(edge)) {
					Object targetPort = model.getTarget(edge);
					Object targetVertex = model.getParent(targetPort);
					VertexView targetVertexView =
						(VertexView) jgraph.getView().getMapping(targetVertex, false);
					fillLevels(levels, (level + 1), targetVertexView);
				}
			}
		}

		if (vecForTheCurrentLevel.size() > gridAreaSize) {
			gridAreaSize = vecForTheCurrentLevel.size();
		}

	}

	/** calculates the minimum for the paint area.
	 *
	 */
	private Point findMinimumAndSpacing(
		CellView[] graphCellViews) {
		try {

			// variables
			/* represents the minimum x and y  value for the paint area
			 */
			int min_x = 1000000;
			int min_y = 1000000;
			
			// find the maximum & minimum coordinates
			for (int i = 0; i < graphCellViews.length; i++) {
				
				// the cellView and their bounds
				CellView cellView = graphCellViews[i];
				Rectangle cellViewBounds = cellView.getBounds();
				
				// checking min area
				min_x = min(cellViewBounds.x, min_x);
				min_y = min(cellViewBounds.y, min_y);
			}
			return new Point(min_x, min_y);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
			 
	
	/**
	 * @param i
	 * @param min_y
	 * @return returns minimum of two numbers
	 */
	private int min(int x, int y) {
		
		if (x < y) {
			return x;
		}
		else {
			return y;
		}
	}

	private void solveEdgeCrosses(Vector levels) {

		int movementsCurrentLoop = -1;

		while (movementsCurrentLoop != 0) {

			// reset the movements per loop count
			movementsCurrentLoop = 0;

			// top down
			for (int i = 0; i < levels.size() - 1; i++) {
				movementsCurrentLoop
					+= solveEdgeCrosses(true, levels, i);
			}

			// bottom up
			for (int i = levels.size() - 1; i >= 1; i--) {
				movementsCurrentLoop
					+= solveEdgeCrosses(false, levels, i);
			}
		}
	}

	private int solveEdgeCrosses(
		boolean down,
		Vector levels,
		int levelIndex) {
		// Get the current level
		Vector currentLevel = (Vector) levels.get(levelIndex);
		int movements = 0;

		// restore the old sort
		Object[] levelSortBefore = currentLevel.toArray();

		// new sort
		Collections.sort(currentLevel);

		movements = getMovements(currentLevel, levelSortBefore);
	

		// Collecations Sort sorts the highest value to the first value
		for (int j = currentLevel.size() - 1; j >= 0; j--) {
			CellWrapper sourceWrapper = (CellWrapper) currentLevel.get(j);

			VertexView sourceView = sourceWrapper.getVertexView();

			Object sourceVertex = sourceView.getCell();
			int sourcePortCount = model.getChildCount(sourceVertex);

			for (int k = 0; k < sourcePortCount; k++) {
				Object sourcePort = model.getChild(sourceVertex, k);

				solveCrossEdgesFromVertex(
						down,
						levelIndex,
						sourceWrapper,
						sourcePort);
			}
		}

		return movements;
	}
	

	private void solveCrossEdgesFromVertex(
		boolean down,
		int levelIndex,
		CellWrapper sourceWrapper,
		Object sourcePort) {
			
		Iterator sourceEdges = model.edges(sourcePort);
		while (sourceEdges.hasNext()) {
			Object edge = sourceEdges.next();
		
			// if it is a forward edge follow it
			Object targetPort = null;
			if (down && sourcePort == model.getSource(edge)) {
				targetPort = model.getTarget(edge);
			}
			if (!down && sourcePort == model.getTarget(edge)) {
				targetPort = model.getSource(edge);
			}
			if (targetPort == null)
				continue;
		
			Object targetCell = model.getParent(targetPort);
			
			VertexView targetVertexView =
				(VertexView) jgraph.getView().getMapping(targetCell,false);
		
			CellWrapper targetWrapper =
				(CellWrapper) targetVertexView.getAttributes().get(CELL_WRAPPER);
		
			// do it only if the edge is a forward edge to a deeper level
			if (down
				&& targetWrapper != null
				&& targetWrapper.getLevel() > levelIndex) {

				targetWrapper.addToEdgeCrossesIndicator(
					sourceWrapper.getEdgeCrossesIndicator());
			}
			if (!down
				&& targetWrapper != null
				&& targetWrapper.getLevel() < levelIndex) {

				targetWrapper.addToEdgeCrossesIndicator(
					sourceWrapper.getEdgeCrossesIndicator());
			}
		}
	
	}

	private int getMovements(
		Vector currentLevel,
		Object[] levelSortBefore) {
		
		int movements = 0;
		
		// test for movements
		for (int j = 0; j < levelSortBefore.length; j++) {
			if (((CellWrapper) levelSortBefore[j]).getEdgeCrossesIndicator()
				!= ((CellWrapper) currentLevel.get(j))
					.getEdgeCrossesIndicator()) {
				movements++;
			}
		}
		return movements;
	}

	private void moveToBarycenter(
		CellView[] allSelectedViews,
		Vector levels) {

		//================================================================
		// iterate any ReViewNodePort
		for (int i = 0; i < allSelectedViews.length; i++) {
			if (!(allSelectedViews[i] instanceof VertexView))
				continue;

			VertexView vertexView = (VertexView) allSelectedViews[i];

			CellWrapper currentwrapper =
				(CellWrapper) vertexView.getAttributes().get(CELL_WRAPPER);

			Object vertex = vertexView.getCell();
			int portCount = model.getChildCount(vertex);

			for (int k = 0; k < portCount; k++) {
				Object port = model.getChild(vertex, k);

				adjustVertexPriority(vertexView, currentwrapper, port);
			}
		}

		//================================================================
		for (int j = 0; j < levels.size(); j++) {
			Vector level = (Vector) levels.get(j);
			for (int i = 0; i < level.size(); i++) {
				// calculate the initial Grid Positions 1, 2, 3, .... per Level
				CellWrapper wrapper = (CellWrapper) level.get(i);
				wrapper.setGridPosition(i);
			}
		}

		int	movementsCurrentLoop = -1;

		while (movementsCurrentLoop != 0) {

			// reset movements
			movementsCurrentLoop = 0;

			// top down
			for (int i = 1; i < levels.size(); i++) {
				movementsCurrentLoop += moveToBarycenter(levels, i);
			}

			// bottom up
			for (int i = levels.size() - 1; i >= 0; i--) {
				movementsCurrentLoop += moveToBarycenter(levels, i);
			}
		}

	}

	private void adjustVertexPriority(
		VertexView vertexView,
		CellWrapper currentwrapper,
		Object port) {
		// iterate any Edge in the port
		
		Iterator edges = model.edges(port);
		while (edges.hasNext()) {
			Object edge = edges.next();
		
			Object neighborPort = getNeighborPort(port, edge);
			
			if (neighborPort == null) {
				continue;
			}
			
			Object neighborVertex = model.getParent(neighborPort);
		
			VertexView neighborVertexView =
				(VertexView) jgraph.getView().getMapping(
					neighborVertex,
					false);
		
			if (neighborVertexView == vertexView)
				continue;
		
			CellWrapper neighborWrapper =
				(CellWrapper) neighborVertexView.getAttributes().get(
					CELL_WRAPPER);
		
			if (currentwrapper == null
				|| neighborWrapper == null
				|| currentwrapper.level == neighborWrapper.level)
				continue;
		
			currentwrapper.priority++;
		
		}
	}

	private Object getNeighborPort(Object port, Object edge) {
		
		Object neighborPort = null;
		// if the Edge is a forward edge we should follow this edge
		if (port == model.getSource(edge)) {
			neighborPort = model.getTarget(edge);
		} else {
			if (port == model.getTarget(edge)) {
				neighborPort = model.getSource(edge);
			} 
		}
		return neighborPort;
	}

	private int moveToBarycenter(
		Vector levels,
		int levelIndex) {

		// Counter for the movements
		int movements = 0;

		// Get the current level
		Vector currentLevel = (Vector) levels.get(levelIndex);

		for (int currentIndexInTheLevel = 0;
			currentIndexInTheLevel < currentLevel.size();
			currentIndexInTheLevel++) {

			CellWrapper sourceWrapper =
				(CellWrapper) currentLevel.get(currentIndexInTheLevel);

			float gridPositionsSum = 0;
			float countNodes = 0;

			VertexView vertexView = sourceWrapper.getVertexView();
			Object vertex = vertexView.getCell();
			int portCount = model.getChildCount(vertex);

			for (int i = 0; i < portCount; i++) {
				Object port = model.getChild(vertex, i);

				Iterator edges = model.edges(port);
				while (edges.hasNext()) {
					
					CellWrapper targetWrapper =
						getTargetCell(levelIndex, sourceWrapper, port, edges.next());
					
					if (targetWrapper != null) {
						gridPositionsSum += targetWrapper.getGridPosition();
						countNodes++;
					}

				}
			}

			//----------------------------------------------------------
			// move node to new x coord
			//----------------------------------------------------------

			if (countNodes > 0) {
				float tmp = (gridPositionsSum / countNodes);
				int newGridPosition = Math.round(tmp);
				boolean toRight =
					(newGridPosition > sourceWrapper.getGridPosition());

				boolean moved = true;

				while (newGridPosition != sourceWrapper.getGridPosition()
					&& moved) {
					int tmpGridPos = sourceWrapper.getGridPosition();

					moved =
						move(
							toRight,
							currentLevel,
							currentIndexInTheLevel,
							sourceWrapper.getPriority());

					if (moved)
						movements++;
				}
			}
		}
		return movements;
	}

	private CellWrapper getTargetCell(
		int levelIndex,
		CellWrapper sourceWrapper,
		Object port,
		Object edge) {
		boolean goodEdge = true;
		
		
		
		// if it is a forward edge follow it
		Object neighborPort = getNeighborPort(port, edge);
		if (neighborPort == null) {
			return null;
		}
		
		Object neighborVertex = model.getParent(neighborPort);
		
		VertexView neighborVertexView =
			(VertexView) jgraph.getView().getMapping(neighborVertex,false);
		CellWrapper targetWrapper =
			(CellWrapper) neighborVertexView.getAttributes().get(CELL_WRAPPER);
					
		if (targetWrapper == null
			|| targetWrapper.getLevel() == levelIndex
			|| targetWrapper == sourceWrapper)
			return null; 
			
		return targetWrapper;
	}

	/**@param  toRight <tt>true</tt> = try to move the currentWrapper to right; <tt>false</tt> = try to move the currentWrapper to left;
	 * @param  currentLevel Vector which contains the CellWrappers for the current level
	 * @param  currentIndexInTheLevel
	 * @param  currentPriority
	 * @param  currentWrapper The Wrapper
	 *
	 * @return The free GridPosition or -1 is position is not free.
	 */
	private boolean move(
		boolean toRight,
		Vector currentLevel,
		int currentIndexInTheLevel,
		int currentPriority) {

		CellWrapper currentWrapper =
			(CellWrapper) currentLevel.get(currentIndexInTheLevel);

		boolean moved = false;
		int neighborIndexInTheLevel =
			currentIndexInTheLevel + (toRight ? 1 : -1);
		int newGridPosition =
			currentWrapper.getGridPosition() + (toRight ? 1 : -1);

		// is the grid position possible?

		if (0 > newGridPosition || newGridPosition >= gridAreaSize) {
			return false;
		}

		// if the node is the first or the last we can move
		if (toRight
			&& currentIndexInTheLevel == currentLevel.size() - 1
			|| !toRight
			&& currentIndexInTheLevel == 0) {

			moved = true;

		} else {
			// else get the neighbor and ask his gridposition
			// if he has the requested new grid position
			// check the priority

			CellWrapper neighborWrapper =
				(CellWrapper) currentLevel.get(neighborIndexInTheLevel);

			int neighborPriority = neighborWrapper.getPriority();

			if (neighborWrapper.getGridPosition() == newGridPosition) {
				if (neighborPriority >= currentPriority) {
					return false;
				} else {
					moved =
						move(
							toRight,
							currentLevel,
							neighborIndexInTheLevel,
							currentPriority);
				}
			} else {
				moved = true;
			}
		}

		if (moved) {
			currentWrapper.setGridPosition(newGridPosition);
		}
		return moved;
	}

	private void drawGraph(
		Vector levels,
		Point min) {
		// paint the graph

		Map viewMap = new Hashtable();
		
		Point spacing = new Point();
		
		spacing.x = HORIZONTAL_SPACING;
		spacing.y = VERTICAL_SPACING;
		
		for (int rowCellCount = 0;
			rowCellCount < levels.size();
			rowCellCount++) {
			Vector level = (Vector) levels.get(rowCellCount);

			for (int colCellCount = 0;
				colCellCount < level.size();
				colCellCount++) {
				CellWrapper wrapper = (CellWrapper) level.get(colCellCount);
				VertexView view = wrapper.vertexView;

				// remove the temp objects
				/* While the Algorithm is running we are putting some
				 *  attributeNames to the MyGraphCells. This method
				 *  cleans this objects from the MyGraphCells.
				 *
				 */
				view.getAttributes().remove(CELL_WRAPPER);
				view.getAttributes().remove(VISITED);
				wrapper.vertexView = null;

				Rectangle bounds = view.getBounds();
				
//			Use this code to lay the table horizontally, instead of vertically 
//				bounds.setLocation( min.x + spacing.x * wrapper.getGridPosition(),
//		                      min.y + spacing.y * rowCellCount);

				bounds.setLocation( min.x + spacing.x * rowCellCount,
		                      min.y + spacing.y * wrapper.getGridPosition());

			}

		}
		
	}

	/** cell wrapper contains all values
	 *  for one node
	 */
	class CellWrapper implements Comparable {

		/** sum value for edge Crosses
		 */
		private double edgeCrossesIndicator = 0;
		/** counter for additions to the edgeCrossesIndicator
		 */
		private int additions = 0;
		/** the vertical level where the cell wrapper is inserted
		 */
		int level = 0;
		/** current position in the grid
		 */
		int gridPosition = 0;
		/** priority for movements to the barycenter
		 */
		int priority = 0;
		/** reference to the wrapped cell
		 */
		VertexView vertexView = null;

		/** creates an instance and memorizes the parameters
		 *
		 */
		CellWrapper(
			int level,
			double edgeCrossesIndicator,
			VertexView vertexView) {
			this.level = level;
			this.edgeCrossesIndicator = edgeCrossesIndicator;
			this.vertexView = vertexView;
			additions++;
		}

		/** returns the wrapped cell
		 */
		VertexView getVertexView() {
			return vertexView;
		}

		/** resets the indicator for edge crosses to 0
		 */
		void resetEdgeCrossesIndicator() {
			edgeCrossesIndicator = 0;
			additions = 0;
		}

		/** retruns the average value for the edge crosses indicator
		 *
		 *  for the wrapped cell
		 *
		 */

		double getEdgeCrossesIndicator() {
			if (additions == 0)
				return 0;
			return edgeCrossesIndicator / additions;
		}

		/** Addes a value to the edge crosses indicator
		 *  for the wrapped cell
		 *
		 */
		void addToEdgeCrossesIndicator(double addValue) {
			edgeCrossesIndicator += addValue;
			additions++;
		}
		/** gets the level of the wrapped cell
		 */
		int getLevel() {
			return level;
		}

		/** gets the grid position for the wrapped cell
		 */
		int getGridPosition() {
			return gridPosition;
		}

		/** Sets the grid position for the wrapped cell
		 */
		void setGridPosition(int pos) {
			this.gridPosition = pos;
		}

		/** increments the the priority of this cell wrapper.
		 *
		 *  The priority was used by moving the cell to its
		 *  barycenter.
		 *
		 */

		void incrementPriority() {
			priority++;
		}

		/** returns the priority of this cell wrapper.
		 *
		 *  The priority was used by moving the cell to its
		 *  barycenter.
		 */
		int getPriority() {
			return priority;
		}

		/**
		 * @see java.lang.Comparable#compareTo(Object)
		 */
		public int compareTo(Object compare) {
			if (((CellWrapper) compare).getEdgeCrossesIndicator()
				== this.getEdgeCrossesIndicator())
				return 0;

			double compareValue =
				(((CellWrapper) compare).getEdgeCrossesIndicator()
					- this.getEdgeCrossesIndicator());

			return (int) (compareValue * 1000);

		}
	}
}

