////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

import com.jgraph.graph.GraphCell;
import com.jgraph.graph.GraphConstants;
import com.jgraph.graph.DefaultGraphCell;


import edu.uiuc.jdbv.schema.Table;
import edu.uiuc.jdbv.schema.Column;

import java.util.*;



/**
 * A GraphCell Model for a Table.
 *  
 * @author Brian Sidharta
 * @version $Revision: 1.6 $
 */
public class TableGraphCell extends DefaultGraphCell
{
    /** The attribute key for the visible columns list */
    public static final String VISIBLE_COLUMNS_KEY = "VisibleColumns";

    /** Comparator that sorts columns, taking into account primary keys */
    private static transient ColumnComparator columnComparator = 
        new ColumnComparator();


    /** the table that this Graph Cell represents */
    private Table table;

    /** The visible columns */
    private List visibleColumnList;



    /**
     * Constructor that takes the table that this Graph Cell represents.
     */
    public TableGraphCell( Table table )
    {
        this( table, GraphConstants.createMap());
    }

    /**
     * Constructor that takes the table that this Graph Cell represents
     * and an attribute map for attributes
     */
    public TableGraphCell( Table table, Map attributeMap )
    {
        if( table == null ) {
            throw new NullPointerException( "Table is null!" );
        }
        this.table = table;

        visibleColumnList = (List)attributeMap.get( VISIBLE_COLUMNS_KEY );
        if (visibleColumnList == null) {
            visibleColumnList = createDefaultVisibleColumnList( table );
            attributeMap.put( VISIBLE_COLUMNS_KEY, visibleColumnList );
        } // end of if ()

        sortVisibleColumns();
        setAttributes( attributeMap );
    }


    /** Returns the table that this cell represents */
    public Table getTable()
    {
        return (table);
    }


    public int hashCode()
    {
        return (table.hashCode());
    }


    public boolean equals( Object o )
    {
        if( o == this ) {
            return (true);
        } else if( o instanceof TableGraphCell ) {
            TableGraphCell graphCell = (TableGraphCell)o;
            return (table.equals( graphCell.table ));
        } else {
            return (false);
        }
    }


    /**
     * Sorts the visible column list so primary keys are at the head
     */
    private void sortVisibleColumns()
    {
        Collections.sort( visibleColumnList, columnComparator );
    }


    /**
     * Retrieves the number of visible columns.
     */
    public int getNumVisibleColumns()
    {
        return (visibleColumnList.size());
    }


    /**
     * Returns whether the supplied column is visible in the table cell
     */
    public boolean isColumnVisible( Column column )
    {
        return (visibleColumnList.contains( column ));
    }


    /**
     * Shows or hides a column in this table cell
     */
    public void setColumnVisible( Column column, boolean visible )
    {
        if( visible && !visibleColumnList.contains( column )) {
            visibleColumnList.add( column );
        } else {
            visibleColumnList.remove( column );
        }
        sortVisibleColumns();

        // TODO Find a way to tell JGraph to redraw the cell!
    }


    /**
     * Return the visible column list.
     * @return a List of Column
     */
    public Column[] getVisibleColumns()
    {
        Column[] cols = new Column[visibleColumnList.size()];
        return ((Column[])visibleColumnList.toArray( cols ));
    }


    /**
     * Creates a list of columns to show by default for the given table
     * @return a List of ColumnS that are in the Table
     */
    protected static List createDefaultVisibleColumnList( Table table )
    {
        Column[] columns = table.getColumns();
        List returnList = new ArrayList( columns.length );

        for( int i = 0; i < columns.length; i++ ) {
            Column col = columns[i];
            if( col.isKey()) {
                returnList.add( col );
            }
        }

        return (returnList);
    }



    /**
     * Comparator that determines the ordering of columns.
     */
    private static class ColumnComparator implements Comparator
    {
        public int compare( Object o1, Object o2 )
        {
            Column c1 = (Column)o1;
            Column c2 = (Column)o2;

            if( c1.isKey() && c2.isKey()) {
                return (compareNames( c1, c2 ));
            } else if( c1.isKey()) {
                // And c2 is not a key
                return (-1);
            } else if( c2.isKey()) {
                // And c1 is not a key
                return (1);
            } else { // Neither is a key
                return (compareNames( c1, c2 ));
            }
        }


        private int compareNames( Column c1, Column c2 )
        {
            String s1 = c1.getName() + c1.getType();
            String s2 = c2.getName() + c2.getType();
            return (s1.compareTo( s2 ));
        }

        public boolean equals( Object obj )
        {
            if( obj instanceof ColumnComparator ) {
                return (true);
            } else {
                return (false);
            }
        }
    }

}

