////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

import edu.uiuc.jdbv.schema.*;

import java.awt.*;
import javax.swing.*;
import java.util.Map;

import com.jgraph.JGraph;
import com.jgraph.graph.*;


/**
 * A JGraph Vertex renderer for Tables
 *  
 * @author Brian Sidharta
 * @version $Revision: 1.8 $
 */
public class TableRenderer extends JPanel implements CellViewRenderer
{
    /** The default foreground color = Color.black */
    public static final Color DEFAULT_FOREGROUND = Color.black;

    public static final Color DEFAULT_BACKGROUND = Color.yellow;

    /** The filename for the key icon = "resources/key.gif" */
    public static final String KEY_ICON_FILENAME = "resources/key.gif";

    /** Our key icon */
    private static final Icon KEY_ICON = new ImageIcon
	( ClassLoader.getSystemResource( KEY_ICON_FILENAME ));

    /** Vertical padding around title */
    public static final int TITLE_PADDING = 3;

    /** Padding around column names */
    public static final int COLUMN_PADDING = 1;

    /** Left margin for column icon */
    public static final int COLUMN_ICON_LEFT = 2;

    /** Left margin for column name text */
    public static final int COLUMN_LEFT = KEY_ICON.getIconWidth() + 2 + 
        COLUMN_ICON_LEFT;

    /** The padding between text rows in pixels */
    public static final int ROW_SPACING = 3;

    /** The padding around the more-content indicator arrow */
    public static final int MORE_ARROW_PADDING = 4;

    /** The width of the more-content indicator arrow */
    public static final int MORE_ARROW_WIDTH = 8;

    /** The height of the more-content indicator arrow */
    public static final int MORE_ARROW_HEIGHT = 8;


    /** Cache the current graph for drawing. */
    transient protected JGraph graph;

    /** Cached hasFocus and selected value. */
    transient protected boolean hasFocus, selected, preview, opaque;

    /** Cached default foreground and default background. */
    transient protected Color defaultForeground, defaultBackground, bordercolor;

    /** Cached table for drawing */
    transient protected Table table;

    
    /** Cached columns for drawing */
    transient protected Column[] columns;


    /** Cached borderwidth. */
    transient protected int borderWidth;


    /** Default constructor */
    public TableRenderer() 
    {
        defaultForeground = DEFAULT_FOREGROUND;
        defaultBackground = DEFAULT_BACKGROUND;
    }


    /**
     * Configure and return the renderer based on the passed in
     * components. The value is typically set from messaging the
     * graph with <code>convertValueToString</code>.
     * We recommend you check the value's class and throw an
     * illegal argument exception if it's not correct.
     *
     * @param   graph the graph that that defines the rendering context.
     * @param   value the object that should be rendered.
     * @param   selected whether the object is selected.
     * @param   hasFocus whether the object has the focus.
     * @param   isPreview whether we are drawing a preview.
     * @return	the component used to render the value.
     */
    public Component getRendererComponent( JGraph graph, CellView cellview,
                                           boolean sel, boolean focus,
                                           boolean preview )
    {
        if (cellview instanceof TableView) {
            TableView view = (TableView) cellview;
            view.recalculateColumnOffsets();

            TableGraphCell tableCell = (TableGraphCell)view.getCell();
            table = tableCell.getTable();
            columns = tableCell.getVisibleColumns();


            this.graph = graph;
            this.hasFocus = focus;
            this.selected = sel;
            this.preview = preview;

            setComponentOrientation(graph.getComponentOrientation());

            if (view.isLeaf())
                installAttributes(view);
            else {
                setBorder(null);
                setOpaque(false);
            }

            return this;
        }
        return null;
    }




    /**
     * Install the attributes of specified cell in this
     * renderer instance. This means, retrieve every published
     * key from the cells hashtable and set global variables
     * or superclass properties accordingly.
     *
     * @param   cell to retrieve the attribute values from.
     */
    protected void installAttributes(CellView view) {
        Map map = view.getAttributes();
        setOpaque(GraphConstants.isOpaque(map));
        setBorder(GraphConstants.getBorder(map));
        bordercolor = GraphConstants.getBorderColor(map);
        borderWidth = Math.max(1, Math.round(GraphConstants.getLineWidth(map)));
        if (getBorder() == null && bordercolor != null)
            setBorder(BorderFactory.createLineBorder(bordercolor, borderWidth));
        Color foreground = GraphConstants.getForeground(map);
        setForeground((foreground != null) ? foreground : defaultForeground);
        Color background = GraphConstants.getBackground(map);
        setBackground((background != null) ? background : defaultBackground);
        setFont(GraphConstants.getFont(map));
    }

    /**
     * Paint the renderer. Overrides superclass paint
     * to add specific painting.
     */
    public void paint(Graphics g) {
        super.paint(g);

        Graphics2D g2 = (Graphics2D)g;
        Dimension dim = getSize();
        int width = dim.width;
        int height = dim.height;

        FontMetrics fm = g.getFontMetrics();
        Font normalFont = g.getFont();
        Font titleFont = normalFont.deriveFont( Font.BOLD );
        int colTextHeight = 
            fm.getMaxAscent() - fm.getMaxDescent() + ROW_SPACING ;
        int yCursor = colTextHeight + TITLE_PADDING;
        int keyIconYOffset = -(KEY_ICON.getIconHeight() + fm.getMaxAscent() -
                               fm.getMaxDescent()) / 2;

        String title = table.getName();
        int xCursor = (width - fm.stringWidth( title )) / 2;

        Stroke lineStroke = new BasicStroke( (float)borderWidth );
        Stroke textStroke = new BasicStroke( 1.0f );

        // Draw the title box with name centered
        g2.setStroke( textStroke );
        g.setColor( bordercolor );
        g.setFont( titleFont );
        g.drawString( title, xCursor, yCursor );

        g2.setStroke( lineStroke );
        yCursor += TITLE_PADDING + borderWidth;
        g.drawLine( 0, yCursor, width, yCursor );

            
        // Draw the columns
        g.setColor( getForeground());
        g2.setStroke( textStroke );
        g.setFont( normalFont );
        for( int i = 0; i < columns.length; i++ ) {
            Column col = columns[i];
            yCursor += ROW_SPACING + colTextHeight;
            if( col.isKey()) {
                KEY_ICON.paintIcon( this, g, COLUMN_ICON_LEFT, 
                                    yCursor + keyIconYOffset );
            }
            g.drawString( col.getName(), COLUMN_LEFT, yCursor );

            // If we're below the the bottom of the box
            if( yCursor > height ) {
                // Draw the more-content arrow
                int right = width - MORE_ARROW_PADDING;
                int left = right - MORE_ARROW_WIDTH;
                int center = right - MORE_ARROW_WIDTH/2;
                int bottom = height - MORE_ARROW_PADDING;
                int top = bottom - MORE_ARROW_HEIGHT;
                int[] xPts = {center, left, right};
                int[] yPts = {bottom, top, top};
                g.fillPolygon( xPts, yPts, 3 );

                break;
            }
        }

        
        

        if (selected || hasFocus) {

            // Draw selection/highlight border
            ((Graphics2D) g).setStroke(GraphConstants.SELECTION_STROKE);
            if (hasFocus)
                g.setColor(graph.getGridColor());
            else if (selected)
                g.setColor(graph.getHighlightColor());
            Dimension d = getSize();
            g.drawRect(0, 0, d.width - 1, d.height - 1);
        }
    }



    /**
     * Returns <code>true</code> if <code>key</code> is a supported
     * attribute in the renderer. Supported attributes affect the
     * visual appearance of the renderer.
     *
     * @param   key the key that defines the attribute to be checked.
     * @return	true if <code>key</code> is supported by this renderer.
     */
    public boolean supportsAttribute(Object key) {
        return key.equals(GraphConstants.BOUNDS)
            || key.equals(GraphConstants.BACKGROUND)
            || key.equals(GraphConstants.BORDER)
            || key.equals(GraphConstants.FONTNAME)
            || key.equals(GraphConstants.FONTSIZE)
            || key.equals(GraphConstants.FONTSTYLE)
            || key.equals(GraphConstants.FOREGROUND)
            || key.equals(GraphConstants.BORDERCOLOR)
            || key.equals(GraphConstants.LINEWIDTH)
            || key.equals(GraphConstants.OPAQUE);
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void validate() {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void revalidate() {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void repaint(long tm, int x, int y, int width, int height) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void repaint(Rectangle r) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    protected void firePropertyChange(
                                      String propertyName,
                                      Object oldValue,
                                      Object newValue) 
    {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   byte oldValue,
                                   byte newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   char oldValue,
                                   char newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   short oldValue,
                                   short newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   int oldValue,
                                   int newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   long oldValue,
                                   long newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   float oldValue,
                                   float newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   double oldValue,
                                   double newValue) {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void firePropertyChange(
                                   String propertyName,
                                   boolean oldValue,
                                   boolean newValue) {
    }

}
