////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.diagram;

import edu.uiuc.jdbv.schema.Column;

import java.awt.*;
import java.util.*;

import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import com.jgraph.JGraph;
import com.jgraph.graph.*;
import com.jgraph.plaf.GraphUI;
import com.jgraph.plaf.basic.BasicGraphUI;

import gnu.trove.*;


/**
 * A view that displays a TableGraphCell
 * @author Brian Sidharta
 * @version $Revision: 1.7 $
 */
public class TableView extends VertexView
{
    /** Distance perimeterPoints should be away from the table */
    public static final int PERIMETER_DISTANCE = 14;


    /** The column offset to use when the column is hidden */
    public static final int COLUMN_HIDDEN_OFFSET = -1;

    /** Renderer for the class. */
    public static TableRenderer renderer = new TableRenderer();

    /** The cached column y-offsets, maps Column to int y-offset */
    private transient TObjectIntHashMap columnOffsetMap = null;

    /** The cached y-offset of the header */
    private transient int headingYOffset = 0;




    /**
     * Constructs a table view for the specified model object and the 
     * specified child views.
     * 
     * @param cell reference to the model object
     */
    public TableView( TableGraphCell cell, JGraph graph, CellMapper cm)
    {
        super( cell, graph, cm );
        Map attribs = getAttributes();
        GraphConstants.setOpaque( attribs, true );

        update();
    }
    

    /**
     * Overrides the parent method to udpate the cached points.
     */
    public void update() {
        super.update();
        bounds = GraphConstants.getBounds(attributes);
        if (isLeaf() && bounds == null) {
            bounds = new Rectangle(10, 10, 200, 100);
            GraphConstants.setBounds(attributes, bounds);
        }

        recalculateColumnOffsets();
    }


    /**
     * Calculates the column y-offsets
     */
    void recalculateColumnOffsets()
    {
        TableGraphCell tableCell = (TableGraphCell)cell;
        Map tableAttribs = tableCell.getAttributes();
        
        Font font = GraphConstants.getFont( tableAttribs );
        Graphics g = getGraph().getGraphics();
        if (g == null) {
            return;
        } // end of if ()
        
        FontMetrics fm = g.getFontMetrics( font );
        int colTextHeight = 
            fm.getMaxAscent() - fm.getMaxDescent() + TableRenderer.ROW_SPACING;


        // Reset all offsets
        Column[] allCols = tableCell.getTable().getColumns();
        headingYOffset = TableRenderer.TITLE_PADDING + 
            TableRenderer.ROW_SPACING + (colTextHeight/2);

        if (columnOffsetMap == null) {
            columnOffsetMap = 
                new TObjectIntHashMap( (int)(1.5*allCols.length) );
        } // end of if ()
        
        for (int i = 0; i < allCols.length; i++ ) {
            Column col = allCols[i];
            columnOffsetMap.put( col, COLUMN_HIDDEN_OFFSET );
        } // end of for ()
        

        // Set yOffset to be halfway in 1st column
        int yOffset = TableRenderer.TITLE_PADDING*2 + colTextHeight*2 - 
            (colTextHeight/2) + TableRenderer.ROW_SPACING + 1;
        Column[] columns = tableCell.getVisibleColumns();

        // Handle visible column offsets
        for (int i = 0; i < columns.length; i++) {
            Column col = columns[i];
            columnOffsetMap.put( col, yOffset );
            yOffset += colTextHeight + TableRenderer.ROW_SPACING;

        } // end of for ()
    }


    /**
     * Retrieves the cached y-offset value for the specified column
     * @return the y-offset or COLUMN_HIDDEN_OFFSET if the column is not
     * visible
     */
    int getYOffsetForColumn( Column col )
    {
        if (columnOffsetMap == null) {
            recalculateColumnOffsets();
        } // end of if ()
        
        return (columnOffsetMap.get( col ));
    }


    /**
     * Retrieves a single y-offset for the specified columns
     */
    int getYOffsetForColumns( Column[] cols )
    {
        // We'll do the average of visible y-offsets; alternatives could be the
        // midpoint or median
        int total = 0;
        int numInTotal = 0;
        for (int i = 0; i < cols.length; i++) {
            int offset = getYOffsetForColumn( cols[i] );
            if (offset != COLUMN_HIDDEN_OFFSET) {
                total += offset;
                numInTotal++;
            } // end of if ()
        } // end of for ()

        if (numInTotal == 0) {
            return (headingYOffset);
        } else {
            return (total / numInTotal);
        } // end of else
        
    }


    /**
     * Retrieves the supplied column point closest to the supplied point
     */
    Point getColumnPoint( Column col, Point point )
    {
        Rectangle bounds = getBounds();
        Point centerPoint = getCenterPoint();
        int x, y;
        y = getYOffsetForColumn( col );
        if (y == COLUMN_HIDDEN_OFFSET) {
            y = headingYOffset;
        } // end of if ()
        
        if (point.x < centerPoint.x) {
            x = bounds.x;
        } else {
            x = bounds.x + bounds.width;
        } // end of else

        y += bounds.y;
        
        return (new Point( x, y ));
    }


    /**
     * Returns the cached bounds for the vertex.
     */
    public Rectangle getBounds() {
        if (!isLeaf())
            return getBounds(getChildViews());
        return bounds;
    }

    /**
     * Returns a renderer for the class.
     */
    public CellViewRenderer getRenderer() {
        return renderer;
    }

    /**
     * Returns the intersection of the bounding rectangle and the
     * straight line between the source and the specified point p.
     * The specified point is expected not to intersect the bounds.
     * <p>
     * Customized to return a point distanced away from the table
     */
    public Point getPerimeterPoint(Point source, Point p) {
        Rectangle bounds = getBounds();
        int x = bounds.x - PERIMETER_DISTANCE;
        int y = bounds.y - PERIMETER_DISTANCE;
        int width = bounds.width + (2*PERIMETER_DISTANCE);
        int height = bounds.height + (2*PERIMETER_DISTANCE);
        int xCenter = (int) (x + width / 2);
        int yCenter = (int) (y + height / 2);
        int dx = p.x - xCenter; // Compute Angle
        int dy = p.y - yCenter;
        double alpha = Math.atan2(dy, dx);
        int xout = 0, yout = 0;
        double pi = Math.PI;
        double pi2 = Math.PI / 2.0;
        double beta = pi2 - alpha;
        double t = Math.atan2(height, width);
        if (alpha < -pi + t || alpha > pi - t) { // Left edge
            xout = x;
            yout = yCenter - (int) (width * Math.tan(alpha) / 2);
        } else if (alpha < -t) { // Top Edge
            yout = y;
            xout = xCenter - (int) (height * Math.tan(beta) / 2);
        } else if (alpha < t) { // Right Edge
            xout = x + width;
            yout = yCenter + (int) (width * Math.tan(alpha) / 2);
        } else { // Bottom Edge
            yout = y + height;
            xout = xCenter + (int) (height * Math.tan(beta) / 2);
        }
        return new Point(xout, yout);
    }



}

