////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.gui;

// import java.awt.*;
// import java.awt.event.*;
// import javax.swing.*;
import javax.swing.event.UndoableEditEvent;
import javax.swing.undo.*;

import net.javaprog.ui.*;

import edu.uiuc.jdbv.schema.Column;
import edu.uiuc.jdbv.diagram.TableGraphCell;
import edu.uiuc.jdbv.JDBV;

/**
 * The ColumnListModel wraps a tableGraphCell so the column list can be
 *  displayed in a swing list component
 *
 * @author Brian Sidharta
 * @author Ross Paul
 * @version $Revision: 1.4 $
 */
public class ColumnListModel extends AbstractCheckBoxListModel
{
    /** The table cell that this model is displaying */
    private TableGraphCell tableCell = null;

    /** Cached columns for speed */
    private Column[] columns = null;


    /**
     * Retrieves the element at a specified index. For ListModel interface.
     *  
     * @param index the 0-based index of the item to retrieve
     * @return Object the data at the specified index
     */
    public Object getElementAt( int index )
    {
        if( tableCell != null ) {
            return (columns[index]);
        } else {
            return (null);
        }
    }



    /**
     * Returns the length of the list. For ListModel interface.
     */
    public int getSize()
    {
        if( tableCell != null ) {
            return (columns.length);
        } else {
            return (0);
        }
    }


	/**
	 * Returns whether the element at the specified position is
	 * checked.
	 *
	 * @param index the position of the element
	 * @return whether the element at <code>index</code> is checked
     * @see net/javaprog/ui/CheckBoxListModel
	 */
	public boolean isElementChecked( int index )
    {
        if( tableCell == null ) {
            return (false);
        } else {
            return (tableCell.isColumnVisible( columns[index] ));
        }
    }

    /**
     * Checks or unchecks the element at the given position.
     *
     * @param tableCell the tabelCell where the column lives
     * @param index the position of the element
     * @param checked whether the element at <code>index</code> should
     *        be checked or not
     * @see net/javaprog/ui/CheckBoxListModel
     */
    public void setColumnVisible(TableGraphCell tableCell, 
				 int index, boolean checked)
    {
        if( tableCell != null ) {
            tableCell.setColumnVisible( columns[index], checked );
        }
        JDBV.getInstance().MAIN_WINDOW.repaint();	
    }

    /**
     * Checks or unchecks the element at the given position, then updates the
     * current undoManager
     *
     * @param index the position of the element
     * @param checked whether the element at <code>index</code> should
     *        be checked or not
     * @see net/javaprog/ui/CheckBoxListModel
     */
    public void setElementChecked(int index,boolean checked)
    {
        if (index == -1) {
            return;
        } // end of if ()
        
	setColumnVisible( tableCell, index, checked );
	JDBV.getInstance().undoableEditHappened
	    ( new UndoableEditEvent( this, new UndoSetColumnVisible
				     ( tableCell, index, checked )));
    }


    /**
     * Returns the number of checked elements in this model.
     *
     * @return the number of checked elements
     * @see net/javaprog/ui/CheckBoxListModel
     */
	public int getCheckedElementsCount()
    {
        if( tableCell == null ) {
            return (0);
        }
        return (tableCell.getNumVisibleColumns());

    }



    /**
     * Sets the table graph cell that this column list is displaying
     */
    public void setTableCell( TableGraphCell tableCell )
    {
        fireIntervalRemoved( this, 0, getSize());
        this.tableCell = tableCell;
        if( tableCell != null ) {
            this.columns = tableCell.getTable().getColumns();
        }
        fireIntervalAdded( this, 0, getSize());
    }


    /** Class which is used to undo table additions */
    private class UndoSetColumnVisible extends AbstractUndoableEdit
    {
	TableGraphCell modifiedCell;
	int index;
	boolean checked;
	String columnName;

	public UndoSetColumnVisible( TableGraphCell tableCell,
				     int index, boolean checked )
	{
	    this.modifiedCell = tableCell;
	    this.index = index;
	    this.checked = checked;
            columnName = columns[index].getName();
           
	}

	public String getPresentationName()
	{
	    if( checked == true)
		return "Show column: " + columnName;
	    else
		return "Hide column: " + columnName;
	}
	
	public void redo()
	{
	    super.redo();
	    setColumnVisible( modifiedCell, index, checked );
	}


	public void undo()
	{
	    super.undo();
	    setColumnVisible( modifiedCell, index, !checked );
	}
    }


}

