////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.gui;

import edu.uiuc.jdbv.diagram.Diagram;
import edu.uiuc.jdbv.diagram.TableGraphCell;

import com.jgraph.event.GraphSelectionListener;
import com.jgraph.event.GraphSelectionEvent;

import java.awt.*;
import javax.swing.*;

import net.javaprog.ui.JCheckBoxList;


/**
 * Panel that displays a table's information and allows the user to
 * modify what components of the table are visible.
 *  
 * @author Brian Sidharta
 * @version $Revision: 1.3 $
 */
public class TableInfoPanel extends JPanel
{
    /** Label that displays the name of the selected item(s) */
    private JLabel selectedNameLabel;

    /** The checkbox list that displays the columns */
    private JCheckBoxList columnList;

    /** The column list model */
    private ColumnListModel columnListModel;

    /** Checkbox list that displays the connections for the current table */
    private JCheckBoxList connectionList;

    /** Model for the connection list */
    private ConnectionListModel connectionListModel;

    /** Listens for graph selection events and notifies this panel */
    private GraphSelectionListener graphSelectionListener;



    /** Default constructor */
    public TableInfoPanel()
    {
        super( new BorderLayout());
        initComponents();

        // Create the graph selection listener
        graphSelectionListener = new GraphSelectionListener() {
                public void valueChanged( GraphSelectionEvent e ) {
                    handleSelectionEvent( e );
                }
            };
    }


    /**
     * Initializes the GUI components
     */
    private void initComponents()
    {
        JPanel headerPanel = new JPanel( new BorderLayout());
        headerPanel.add( new JLabel( " Selected: " ), BorderLayout.WEST );
        selectedNameLabel = new JLabel( "" );
        headerPanel.add( selectedNameLabel, BorderLayout.CENTER );

        JPanel mainPanel = new JPanel( new GridLayout( 2, 1 ));

        JPanel columnsPanel = new JPanel( new GridLayout( 1, 1 ));
        columnsPanel.setBorder( BorderFactory.createTitledBorder( "Columns" ));
        mainPanel.add( columnsPanel );

        columnListModel = new ColumnListModel();
        columnList = new JCheckBoxList( columnListModel );
        columnList.setCellRenderer( new ColumnListRenderer());
        JScrollPane colScroller = new JScrollPane( columnList );
        colScroller.setVerticalScrollBarPolicy( JScrollPane.
                                                VERTICAL_SCROLLBAR_ALWAYS );
        columnsPanel.add( colScroller );
        

        JPanel connectionsPanel = new JPanel( new GridLayout( 1, 1 ));
        connectionsPanel.setBorder(BorderFactory.
                                   createTitledBorder("Connected Tables"));
        mainPanel.add( connectionsPanel );

        connectionListModel = new ConnectionListModel();
        connectionList = new JCheckBoxList( connectionListModel );
        connectionList.
            setCellRenderer( new ConnectionListRenderer( connectionListModel ));
        JScrollPane connScroller = new JScrollPane( connectionList );
        connScroller.setVerticalScrollBarPolicy( JScrollPane.
                                                 VERTICAL_SCROLLBAR_ALWAYS );
        connectionsPanel.add( connScroller );


        add( headerPanel, BorderLayout.NORTH );
        add( mainPanel, BorderLayout.CENTER );
    }




    /**
     * Retrieves a GraphSelectionListener that should be registered with
     * the current diagram.
     */
    public GraphSelectionListener getGraphSelectionListener()
    {
        return (graphSelectionListener);
    }


    /**
     * Handles when the selection cells in the diagram
     */
    private void handleSelectionEvent( GraphSelectionEvent e )
    {
        Diagram diagram = (Diagram)e.getSource();
        Object[] cells = diagram.getSelectionCells();
        if( cells.length != 1 ) {
            selectedNameLabel.setText( "(" + cells.length + " objects)" );
            columnListModel.setTableCell( null );
            columnList.setEnabled( false );
            connectionListModel.setTableCellAndDiagram( null, null );
            connectionList.setEnabled( false );

        } else if( cells[0] instanceof TableGraphCell ) {
            // Just one table cell selected
            TableGraphCell tableCell = (TableGraphCell)(cells[0]);
            selectedNameLabel.setText( tableCell.getTable().getName() );

            columnListModel.setTableCell( tableCell );
            columnList.setEnabled( true );
            connectionListModel.setTableCellAndDiagram( tableCell, diagram );
            connectionList.setEnabled( true );
        } else { // Not a table cell - don't display info
            selectedNameLabel.setText( "(" + cells.length + " object)" );
            columnListModel.setTableCell( null );
            columnList.setEnabled( false );
            connectionListModel.setTableCellAndDiagram( null, null );
            connectionList.setEnabled( false );
        }            
    }

}

