////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.imports;

import edu.uiuc.jdbv.schema.*;

import java.util.*;

/**
 * A cache of PendingConnectionS.  This cache handles maintaining a minimum
 * set of data so an optimized set of connections can be constructed.
 *
 * @author Brian Sidharta
 * @version $Revision: 1.2 $
 * @see PendingConnection
 */
public class PendingConnectionCache
{
    /** Maps PendingConnection to themselves, kinda a hack */
    Map pendConnMap = new HashMap(53);


    public PendingConnectionCache()
    {
        
    }

    /**
     * Adds connection columns to the cache, checking for duplicate or existing
     * data before adding in.  This is a safe but somewhat slow method, since
     * it assures that it doesn't overwrite any existing data, but instead
     * adds to it.
     */
    public void addConnection( String pkTable, String pkCol, 
                               String fkTable, String fkCol )
    {
        PendingConnection pendConn = getPendingConnection( pkTable, fkTable );
        if ( pendConn == null) {
            pendConn = new PendingConnection( pkTable, fkTable );
            pendConnMap.put( pendConn, pendConn );
        } // end of if ()
        
        if (!pendConn.containsPKColumn( pkCol )) {
            pendConn.addColumns( pkCol, fkCol );
        } // end of if ()
    }


    /**
     * Retrieves a pending connection for the supplied primary-key-table-name
     * and foreign-key-table-name.
     */
    public PendingConnection getPendingConnection( String pkTable, 
                                                   String fkTable )
    {
        PendingConnection searchConn = 
            new PendingConnection( pkTable, fkTable );
        PendingConnection foundConn = 
            (PendingConnection)pendConnMap.get( searchConn );

        return (foundConn);
    }


    /**
     * Adds a connection, overwriting any previous connection with the same
     * pkTable and fkTable
     */
    public void addConnection( PendingConnection pendConn )
    {
        pendConnMap.put( pendConn, pendConn );
    }


    /**
     * Creates all pending connections in this cache
     */
    public void createAllConnections( Schema schema )
    {
        for (Iterator it = pendConnMap.keySet().iterator(); it.hasNext(); ) {
            PendingConnection conn = (PendingConnection)it.next();
            Table pkTable = schema.getTable( conn.getPKTable());
            Table fkTable = schema.getTable( conn.getFKTable());
            Column[] pkCols = getColumns( conn.getPKCols(), pkTable );
            Column[] fkCols = getColumns( conn.getFKCols(), fkTable );
            Connection.linkTables( pkTable, fkTable, pkCols, fkCols );
           
        } // end of for ()
    }


    private static Column[] getColumns( String[] colNames, Table table )
    {
        Column[] cols = new Column[colNames.length];
        for (int i = colNames.length - 1; i >= 0; --i) {
            cols[i] = table.getColumn( colNames[i] );
        } // end of for ()
        return (cols);
    }
}

