////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.schema;

import java.util.Arrays;

/**
 * This class is the Schema's representation of foreign key relations.
 * 
 *
 * @author Ross Paul
 * @author Brian Sidharta
 * 
 */
public class Connection{

    /** Table whose records are not dependant on the records from the 
     *  dependantTable */
    private final Table independantTable;

    /** Table whose entries require records in the independant table */
    private final Table dependantTable;
    
    /** columns from the independant table which are not dependant on the 
     *	records from the dependantTable */
    private final Column[] independantColumns;

    /** Columns from the dependant table which require records 
     *  in the independant table */
    private final Column[] dependantColumns;


    /** Called by the static linkTables to make a new instance */
    public Connection(Table independantTable, Table dependantTable,
		      Column[] independantColumns, Column[] dependantColumns)
    {
	this.independantTable = independantTable;
	this.dependantTable = dependantTable;
	this.independantColumns = independantColumns;
	this.dependantColumns = dependantColumns;
    }


    /** This method creates a connection and adds it to the tables connections
     *  lists.  
     *  @param independantTable The table being connected to
     *  @param dependantTable Connecting from this table
     *  @param independantColums columns from the independant table
     *  involved in the relation
     *  @param dependantTable columns in the dependant table 
     *  involved in the relation
     *  @throws IllegalArgumentException if
     *          independantColumns.length != dependantColumns.length
     */
    public static Connection linkTables( Table independantTable, 
                                         Table dependantTable,
                                         Column[] independantColumns, 
                                         Column[] dependantColumns )
    {
        if (independantColumns.length != dependantColumns.length) {
            throw new IllegalArgumentException( "Connection creation error: " +
                                                "Column lists have different " +
                                                "column counts: " +
                                                independantColumns.length + " != " +
                                                dependantColumns.length );
        } // end of if ()
        

	Connection connection = new Connection
	    (independantTable, dependantTable,
	     independantColumns,dependantColumns);
	
	independantTable.addEndingConnection(connection);
	dependantTable.addStartingConnection(connection);
	return connection;
    }

    /**	Returns the independantTable */
    public Table getIndependantTable(){
	return independantTable;
    }

    /** Returns the dependantTable */
    public Table getDependantTable(){
	return dependantTable;
    }

    /** Returns the independantColumns */
    public Column[] getIndependantColumns(){
	return independantColumns;
    }

    /** Returns the dependantColumns */
    public Column[] getDependantColumns(){
	return dependantColumns;
    }

    /** Checks to see if this connection is loggically similar to another */
    public boolean equals(Object o)
    {
	if( !(o instanceof Connection) )
	    return false;
	else
	{
	    Connection other = (Connection)o;
	    return ((Arrays.equals(independantColumns,
				   other.getIndependantColumns())) &&
		    (Arrays.equals(dependantColumns,
				   other.getDependantColumns())));
	}
    }
	  

    /** overrides the toString method to allow logical viewing */
    public String toString(){
	StringBuffer buf = new StringBuffer();
	buf.append("\t\tFrom: " + dependantTable.getName());
	buf.append("\tTo: " + independantTable.getName());
	for(int i = 0; i < independantColumns.length; i++){
	    buf.append("\n\t\t      " + dependantColumns[i].getName());
	    buf.append("\t-->\t" + independantColumns[i].getName());
	}
	buf.append("\n");
	return buf.toString();
    }

    
}
