////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.schema;

import java.util.*;
import edu.uiuc.jdbv.util.*;

/**
 * The schema is dbViZ's internal representation of a database's schema.
 * It is made up of tables, columns, and connections representing the 
 * foreign keys which define the dependencies between tables.
 *
 * @author Ross Paul
 * @version $Revision: 1.7 $
 */
public class Schema {
    
    /** Arbitrary name for this schema (database name?) */
    private final String name;

    /** The tables which make up the schema.  Name indexed in a Hashtable 
        for faster lookups */
    private Hashtable tables;

    /** The constructor taks only a name.  Tables should be adde via
     *  addTable(); */
    public Schema(String name)
    {
	this.name = name;
	tables = new Hashtable();
    }

    /** Add a table to the shema 
     *  @param table Table to be added */
    public void addTable(Table table)
    {
	tables.put(table.getName().toUpperCase(),table);
    }

    /** Look up a table by name 
     *  @param name The name of the table
     *  @return The table who's name was searched for */
    public Table getTable(String name)
    {
	return (Table)tables.get(name.toUpperCase());
    }


    /** Returns an array of all the tables in the schema */
    public Table[] getTables()
    {
	Collection c = tables.values();
	Table[] tableArray = new Table[c.size()];
	return (Table[])c.toArray(tableArray);
    }

    /** returns a mapping of table names to tables */
    public Map getTableMap()
    {
        return tables;
    }


    /** Returns the name of the Schema */
    public String getName()
    {
	return name;
    }

    /** overrides the toString method to allow logical viewing */
    public String toString(){
	StringBuffer buf = new StringBuffer();
	buf.append("Schema: " + name);
	buf.append("\n   Tables -\n");
	for(Enumeration enum = tables.elements();enum.hasMoreElements();){
	    buf.append(enum.nextElement());
	}
	return buf.toString();
    }

    /** checks to see if this Schema has the same contents as another */
    public boolean equals(Object o)
    {
        if ( !( o instanceof Schema ))
        {
            return false;
        }
        else
        {
            Schema other = (Schema)o;
            return (CollectionUtils.mapsEqual(tables, other.getTableMap()));
        }
    }


    /**
     * A dummy function added to aid in using the PersistenceDelegates.
     * This is a no-op!
     */
    public void addConnection( Connection conn )
    {
        // NOOP
    }


    /** this code is used for testing.  It builds a simple schema */
    public static void main(String args[]){
	Schema schema = new Schema("Test Schema");
	Table table;
	Column column;

	table = new Table("indi");
	column = new Column("Dr.", "varChar2", false);
	table.addColumn(column);
	column = new Column("Henry", "varChar2", false);
	table.addColumn(column);
	column = new Column("Jones", "varChar2", false);
	table.addColumn(column);
	column = new Column("Junior", "varChar2", true);
	table.addColumn(column);
	schema.addTable(table);

	table = new Table("dep");
	column = new Column("Ferris", "varChar2", true);
	table.addColumn(column);
	column = new Column("Buler's", "varChar2", false);
	table.addColumn(column);
	column = new Column("Day", "varChar2", false);
	table.addColumn(column);
	column = new Column("Off", "varChar2", true);
	table.addColumn(column);
	schema.addTable(table);

	//I know this is horrible, I did it to test various aspects
	Connection.linkTables
	    (schema.getTable("indi"),schema.getTable("dep"),
	     new Column[]{schema.getTable("indi").getColumn("junior"),
			  schema.getTable("indi").getColumn("jones")},
	     new Column[]{schema.getTable("dep").getColumn("ferris"),
			  schema.getTable("dep").getColumn("buler's")});

	System.out.println(schema);
    }
}
