////////////////////////////////////////////////////////////////////////////
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////
package edu.uiuc.jdbv.util;

import java.io.File;

/**
 * A FileFilter that shows only files that end in supplied file extensions.
 * Comparisons are case insensitive.
 *
 * @version $Revision: 1.2 $
 * @author Brian Sidharta
 */
public class SuffixFileFilter extends javax.swing.filechooser.FileFilter
{
    private final String[] EXTENSIONS;

    private String description;

    /**
     * Constructor that takes the file type description, such as 
     * "Image Files"
     * and an array of suffixes to match.
     * @param description a description for the file type
     * @param extensions filename extensions that will be matched, such as:
     *        {"gif","jpg","jpeg","png"}
     */
    public SuffixFileFilter( String description, String[] extensions )
    {
        EXTENSIONS = makeAllLowercase( extensions );
        this.description = createFullDescription( description, EXTENSIONS );
    }

    public String getDescription()
    {
        return (description);
    }

    public boolean accept(File f)
    {
        // Show all subdirectories
        if (f.isDirectory()) {
            return (true);
        } // end of if ()
            
        String filename = f.getName().toLowerCase();
        for (int i = 0; i < EXTENSIONS.length; i++) {
            if (filename.endsWith( EXTENSIONS[i] )) {
                return (true);
            } // end of if ()
        } // end of for ()

        return (false);
    }


    /**
     * Creates a copy of the supplied string array with all strings lowercase
     */
    private static String[] makeAllLowercase( String[] strings )
    {
        String[] returnArray = new String[strings.length];
        for (int i = 0; i < strings.length; i++) {
            returnArray[i] = strings[i].toLowerCase();
        } // end of for ()
        return (returnArray);
    }


    /**
     * Takes a short description and extensions and combines them into
     * a description of the form: desc (*.exts[0], *.exts[1], ..., *.ext[n])
     * @param exts an array of length 1 or more
     */
    private static String createFullDescription( String desc, String[] exts )
    {
        StringBuffer sb = new StringBuffer( desc );
        sb.append( " (*." );
        sb.append( exts[0].toLowerCase() );
        for (int i = 1; i < exts.length; i++) {
            sb.append( ", *." );
            sb.append( exts[i] );
        } // end of for ()
        sb.append( ")" );

        return (sb.toString());
    }


}
	    

